/*

 File: calccontrol.cpp, 2003/06/17

 ACETAF, Version 3.0.1

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "calccontrol.h"

#include "stringutil.h"

void Calculation::setNext()
{
  if( ++counter < oabdat.size() )
    {
      actMethod = oabdat[ counter ]->mth;
      emit activateOutputfor( actMethod, 0 );

      //Save taylorcoefficients
      if( ( actMethod->identity() == Method2 )||( actMethod->identity() == Method4 ) )
	if( taylorcoeff )
	  (*taylorcoeff) = ((MethodTwoAndFour*)actMethod)->Tcoeff();
	else
	  {
	    taylorcoeff = new TaylorMemory<CInterval>( ((MethodTwoAndFour*)actMethod)->Tcoeff() );
	  }

      if( !otodat.empty() && actMethod->identity() == Method2 && otodat[ 0 ] != 0 )
	{
	  if( (otodat[0]->optimization) )
	    {
	      QString mess("Optimizing the order of the Taylor polynomial\nwith ");
	      mess += QString::number( otodat[ 0 ]->segments );
	      mess += " test segments:\n"; 
	      emit sendMessage( mess );
	  
	      otores->tm = taylorcoeff;
	      counter24 = 0;
	      
	      oto = new OptTaylOrd( otodat[ 0 ], otores, (MethodTwoAndFour*)actMethod, shd );
	      oto->start();
	    }
	  else
	    {
	      oab = new OptimAlgB( oabdat[ counter ], oabres, garbage, results, shd );//use OptimAlgB
	      oab->start();
	    }
	}
      else
	if( !otodat.empty() && actMethod->identity() == Method4 && otodat[ 1 ] != 0 )
	  {
	    if( (otodat[1]->optimization) )
	      {
		QString mess("Optimizing the order of the Taylor polynomial\nwith ");
		mess += QString::number( otodat[ 1 ]->segments );
		mess += " test segments:\n"; 
		emit sendMessage( mess );
		
		otores->tm = taylorcoeff;
		counter24 = 1;
		
		oto = new OptTaylOrd( otodat[ 1 ], otores, (MethodTwoAndFour*)actMethod, shd );
		oto->start();
	      }
	    else
	      {
		oab = new OptimAlgB( oabdat[ counter ], oabres, garbage, results, shd );//use OptimAlgB
		oab->start();
	      }
	  }
	else
	  {
	    oab = new OptimAlgA( oabdat[ counter ], oabres, garbage, results, shd );
	    oab->start();
	  }
    }
  else
    {
      killTimers();
      emit calcFinished( taylorcoeff );
    }
}


Calculation::Calculation(const std::vector<OptimData*>& od,
			 bool gw                       ) : oabdat     ( od    ), 
			                                   oto        ( 0     ), 
			                                   otores     ( 0     ), 
 			                                   counter    ( -1    ), 
			                                   garbage    ( gw    ),
			                                   results    ( false ),
			                                   taylorcoeff( 0     )
{
  oabres = new OptimResult;
  shd    = new SharedData;
}

Calculation::Calculation(const std::vector<OptimData*>& od,
			 const std::vector<OptTaylData*>& otdat,
			 bool gw                       ) : oab        ( 0      ),
							   oabdat     ( od     ), 
			                                   oto        ( 0      ),
			                                   otodat     ( otdat  ),
			                                   counter    ( -1     ),
   			                                   garbage    ( gw     ),
							   results    ( false  ),
			                                   taylorcoeff( 0      )
{
  oabres = new OptimResult;
  otores = new OptTaylResult;
  shd    = new SharedData;
}

Calculation::~Calculation()
{
  if( oab         ) delete oab;
  if( oto         ) delete oto;
  if( oabres      ) delete oabres;
  if( otores      ) delete otores;
  if( shd         ) delete shd;
  if( taylorcoeff ) delete taylorcoeff;
  for(int i=0; i < oabdat.size(); i++) delete oabdat[i];
  if( !otodat.empty() )
    {
      delete otodat[0];
      delete otodat[1];
    }
}

void Calculation::start()
{
  startTimer( 100 );
  setNext();
}

void Calculation::stop()
{
  shd->setStatus( Thread::STOP );
}

void Calculation::timerEvent(QTimerEvent*)
{
  if( oab ) 
    {
      if( shd->isThereGarbage() ) 
	{
	  QString text;
	  shd->readGarbage( text );
	  emit sendGarbage( text );
	}
      if( !(oab->running()) )
	{
	  delete oab; oab = 0;
	  switch( shd->readStatus() )
	    {
	    case Thread::ERROR:
	      {
		killTimers();
		QString text;
		shd->readError( text );
		emit sendError( text );
		return;
	      }
	    case Thread::STOP:
	      {
		killTimers();
		QString text;
		shd->readMessage( text );
		emit sendMessage( text );
		emit calcStopped();
		return;
	      }
	    case Thread::TIMEOVER:
	      {
		QString text;
		shd->readMessage( text );
		emit sendMessage( text );
		if( results ) OptimAlgReady();
		shd->setStatus( Thread::WORKING ); //Go on with other methods
		break;
	      }
	    default:
	      {
		OptimAlgReady();
	      }
	    }
	}
    }
  if( oto ) 
    {
      if( shd->isThereAMessage() ) 
	{
	  QString text;
	  shd->readMessage( text );
	  emit sendMessage( text );
	}
      if( !(oto->running()) )
	{
	  delete oto; oto = 0;
	  switch( shd->readStatus() )
	    {
	    case Thread::ERROR:
	      {
		killTimers();
		QString text;
		shd->readError( text );
		emit sendError( text );
		return;
	      }
	    case Thread::STOP:
	      {
		killTimers();
		QString text;
		shd->readMessage( text );
		emit sendMessage( text );
		emit calcStopped();
		return;
	      }
	    default:
	      {
		OptTaylReady();
	      }
	    }
	}
    }
  if( !oab && !oto ) setNext();
}

void Calculation::OptTaylReady()
{
  double usedTime( (otores->optTime) * (otodat[ counter24 ]->maxSegments)/(otodat[ counter24 ]->segments) );

  if( (0 < (oabdat[ counter ]->maxTime)) && ((oabdat[ counter ]->maxTime) < usedTime) )
    {
      QString mess("Estimated computation time is larger than maximal computation time.\n Computation aborted.\n");
      mess += "Estimated computation time: ";
      mess += double2QStr( usedTime, "% .8lf" );

      emit sendMessage( mess );
    }
  else
    {
      oab = new OptimAlgB( oabdat[ counter ], oabres, garbage, results, shd );//use OptimAlgB
      oab->start();
    }
}

void Calculation::OptimAlgReady()
{
  switch( actMethod->identity() )
    {
    case Method2:
      {
	oabres->PolyordMth2 = ((MethodTwoAndFour*)actMethod)->getPolyord();
	break;
      }
    case Method4:
      {
	oabres->PolyordMth4 = ((MethodTwoAndFour*)actMethod)->getPolyord();
	break;
      }
    }

  emit activateOutputfor( actMethod, oabres );
}

#include "calccontrol.moc"

/*

  End of File: calccontrol.cpp

*/


