/*

 File: circle.h, 2003/06/17

 ACETAF, Version 3.0.1

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _CIRCLE_H_INCLUDED
#define _CIRCLE_H_INCLUDED

#include <complex>
#include <cmath>

typedef std::complex<double> Complex;

//Circle with complex midpoint
class Circle
{
  Complex center;
  double radius;

 public:

  Circle() {}
  Circle(const Complex& c, const double& r) : center( c ), radius( std::abs(r) ) {}
  Circle(const Circle& C) : center( C.center ), radius( C.radius ) {}
  ~Circle() {}

  Circle& operator = (const Circle& C)
  {
    center = C.center;
    radius = C.radius;

    return *this;
  }

  const double&  getRadius() const { return radius; }
  const Complex& getCenter() const { return center; }

  void setRadius(const double& r) { radius = std::abs(r); };
 
  void zoom(const double& f) 
    { 
      if( f > 0.0 ) radius *= f;
    }
};

bool operator <  (const Circle&,const Circle&);
bool operator <= (const Circle&,const Circle&);
bool operator == (const Circle&,const Circle&);

#endif

/*

  End of File: circle.h

*/
