/*

 File: functions.cpp, 2004/09/29

 ACETAF, Version 3.0.1

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "functions.h"

// Parameter: CInterval
//
#include "cimath.h" //Use CoStLy functions.

template<> CInterval EXP   (const CInterval& z)                      { return CoStLy::exp(z);     }
template<> CInterval LN    (const CInterval& z)                      { return CoStLy::Ln(z);      }
template<> CInterval SQR   (const CInterval& z)                      { return CoStLy::sqr(z);     }
template<> CInterval SQRT  (const CInterval& z)                      { return CoStLy::Sqrt(z);    }
template<> CInterval SIN   (const CInterval& z)                      { return CoStLy::sin(z);     }
template<> CInterval COS   (const CInterval& z)                      { return CoStLy::cos(z);     }
template<> CInterval TAN   (const CInterval& z)                      { return CoStLy::tan(z);     }
template<> CInterval COT   (const CInterval& z)                      { return CoStLy::cot(z);     }
template<> CInterval ASIN  (const CInterval& z)                      { return CoStLy::Asin(z);    }
template<> CInterval ACOS  (const CInterval& z)                      { return CoStLy::Acos(z);    }
template<> CInterval ATAN  (const CInterval& z)                      { return CoStLy::Atan(z);    }
template<> CInterval ACOT  (const CInterval& z)                      { return CoStLy::Acot(z);    }
template<> CInterval SINH  (const CInterval& z)                      { return CoStLy::sinh(z);    }
template<> CInterval COSH  (const CInterval& z)                      { return CoStLy::cosh(z);    }
template<> CInterval TANH  (const CInterval& z)                      { return CoStLy::tanh(z);    }
template<> CInterval COTH  (const CInterval& z)                      { return CoStLy::coth(z);    }
template<> CInterval ASINH (const CInterval& z)                      { return CoStLy::Asinh(z);   }
template<> CInterval ACOSH (const CInterval& z)                      { return CoStLy::Acosh(z);   }
template<> CInterval ATANH (const CInterval& z)                      { return CoStLy::Atanh(z);   }
template<> CInterval ACOTH (const CInterval& z)                      { return CoStLy::Acoth(z);   }
template<> CInterval SQRT  (const CInterval& z,int n)                { return CoStLy::Sqrt(z,n);  }
template<> CInterval POWER (const CInterval& z,int n)                { return CoStLy::power(z,n); }
template<> CInterval POW   (const CInterval& z1,const CInterval& z2) { return CoStLy::Pow(z1,z2); }

// Parameter: Interval
//
#include "adaptintval.h"

template<> Interval EXP   (const Interval& x)                     { return Adapt::exp(x);     }
template<> Interval LN    (const Interval& x)                     { return Adapt::ln(x);      }
template<> Interval SQR   (const Interval& x)                     { return Adapt::sqr(x);     }
template<> Interval SQRT  (const Interval& x)                     { return Adapt::sqrt(x);    }
template<> Interval SIN   (const Interval& x)                     { return Adapt::sin(x);     }
template<> Interval COS   (const Interval& x)                     { return Adapt::cos(x);     }
template<> Interval TAN   (const Interval& x)                     { return Adapt::tan(x);     }
template<> Interval COT   (const Interval& x)                     { return Adapt::cot(x);     }
template<> Interval ASIN  (const Interval& x)                     { return Adapt::asin(x);    }
template<> Interval ACOS  (const Interval& x)                     { return Adapt::acos(x);    }
template<> Interval ATAN  (const Interval& x)                     { return Adapt::atan(x);    }
template<> Interval ACOT  (const Interval& x)                     { return Adapt::acot(x);    }
template<> Interval SINH  (const Interval& x)                     { return Adapt::sinh(x);    }
template<> Interval COSH  (const Interval& x)                     { return Adapt::cosh(x);    }
template<> Interval TANH  (const Interval& x)                     { return Adapt::tanh(x);    }
template<> Interval COTH  (const Interval& x)                     { return Adapt::coth(x);    }
template<> Interval ASINH (const Interval& x)                     { return Adapt::asinh(x);   }
template<> Interval ACOSH (const Interval& x)                     { return Adapt::acosh(x);   }
template<> Interval ATANH (const Interval& x)                     { return Adapt::atanh(x);   }
template<> Interval ACOTH (const Interval& x)                     { return Adapt::acoth(x);   }
template<> Interval SQRT  (const Interval& x,int n)               { return Adapt::sqrt(x,n);  }
template<> Interval POWER (const Interval& x,int n)               { return Adapt::power(x,n); }
template<> Interval POW   (const Interval& x1,const Interval& x2) { return Adapt::pow(x1,x2); }

// Parameter: CITaylor
//
#include "taylor.h"

template<> CITaylor EXP   (const CITaylor& x)                     { return exp(x);     }
template<> CITaylor LN    (const CITaylor& x)                     { return ln(x);      }
template<> CITaylor SQR   (const CITaylor& x)                     { return sqr(x);     }
template<> CITaylor SQRT  (const CITaylor& x)                     { return sqrt(x);    }
template<> CITaylor SIN   (const CITaylor& x)                     { return sin(x);     }
template<> CITaylor COS   (const CITaylor& x)                     { return cos(x);     }
template<> CITaylor TAN   (const CITaylor& x)                     { return tan(x);     }
template<> CITaylor COT   (const CITaylor& x)                     { return cot(x);     }
template<> CITaylor ASIN  (const CITaylor& x)                     { return asin(x);    }
template<> CITaylor ACOS  (const CITaylor& x)                     { return acos(x);    }
template<> CITaylor ATAN  (const CITaylor& x)                     { return atan(x);    }
template<> CITaylor ACOT  (const CITaylor& x)                     { return acot(x);    }
template<> CITaylor SINH  (const CITaylor& x)                     { return sinh(x);    }
template<> CITaylor COSH  (const CITaylor& x)                     { return cosh(x);    }
template<> CITaylor TANH  (const CITaylor& x)                     { return tanh(x);    }
template<> CITaylor COTH  (const CITaylor& x)                     { return coth(x);    }
template<> CITaylor ASINH (const CITaylor& x)                     { return asinh(x);   }
template<> CITaylor ACOSH (const CITaylor& x)                     { return acosh(x);   }
template<> CITaylor ATANH (const CITaylor& x)                     { return atanh(x);   }
template<> CITaylor ACOTH (const CITaylor& x)                     { return acoth(x);   }
template<> CITaylor SQRT  (const CITaylor& x,int n)               { return sqrt(x,n);  }
template<> CITaylor POWER (const CITaylor& x,int n)               { return power(x,n); }
template<> CITaylor POW   (const CITaylor& x1,const CITaylor& x2) { return pow(x1,x2); }

/*

  End of File: functions.cpp

*/
