/*

 File: gui_mainwindow.cpp, 2008/09/18

 ACETAF, Version 3.0.1

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "gui_mainwindow.h"
#include "analyticcontrol.h"
#include "gui_taylorcoeffdialog.h"

#include "calccontrol.h"
#include "gui_inputwidget.h"
#include "gui_outputwindow.h"
#include "gui_garbagewindow.h"
#include "stringutil.h"
#include "mathexpr.h"
#include "functions.h"

#include <qmultilinedit.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qsplitter.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qfiledialog.h>
#include <qcheckbox.h>
#include <qtoolbutton.h>
#include <qkeycode.h>
#include <qfile.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwidgetstack.h> 
#include <qregexp.h> 
#include <qtextstream.h>

#include "pixmaps/fileopen.xpm"
#include "pixmaps/filesave.xpm"
#include "pixmaps/filesaveas.xpm"
#include "pixmaps/fileprint.xpm"
#include "pixmaps/startexec.xpm"
#include "pixmaps/stopexec.xpm"
#include "pixmaps/analyticity.xpm"
#include "pixmaps/tayloronly.xpm"

#include <stdlib.h>

using Adapt::Interval;
using Adapt::sup;

enum { 
  NMth = 0,
  NFct = 1,
  INC  = 2,
  MII  = 3,
  MIII = 4,
  MIV  = 5,
  MIIV = 6,
  TO   = 7,
  OP   = 8, 
  MO   = 9,
  WS   = 10 
};

const QString wrongInputMessageText[11] = { 
  "No method chosen. :-(",
  "No function. :-(",
  "Input not complete. :-(",
  "Method II: Input not complete. :-(",
  "Method III: Input not complete. :-(",
  "Method IV: Input not complete. :-(",
  "Taylor order optimization: Input not complete. :-(",
  "Taylor order must be greater or equal one. :-(",
  "Optimization parameters not complete. :-(",
  "Orders larger than 150 are not supported. :-(",
  "The number of test segments must be lower\nthan the maximal number of segments. :-(" 
};

extern const QString defaultValues[36] = {
  "exp(z)",                                                   //Expression
  "0,0",                                                      //Center
  "1",                                                        //Radius
  "20",                                                       //Taylor order MII
  "20",                                                       //Taylor order MIV
  "1",                                                        //Precision MII
  "1",                                                        //Precision MIV
  "50",                                                       //Derive order MIII
  "30",                                                       //Derive order MIV
  "128",                                                      //Test segments MII
  "128",                                                      //Test segments MIV
  "100",                                                      //Max. Taylor order MII
  "100",                                                      //Max. Taylor order MIV
  "1024",                                                     //Max. no. of segments MI
  "8192",                                                     //Max. no. of segments MII
  "1024",                                                     //Max. no. of segments MIII
  "8192",                                                     //Max. no. of segments MIV
  "32",                                                       //Min. no. of segments MI
  "32",                                                       //Min. no. of segments MII
  "32",                                                       //Min. no. of segments MIII
  "32",                                                       //Min. no. of segments MIV
  "0.1",                                                      //Max. relative error MI
  "0.1",                                                      //Max. relative error MII
  "0.1",                                                      //Max. relative error MIII
  "0.1",                                                      //Max. relative error MIV
  "0",                                                        //Max. absolute error MI
  "0",                                                        //Max. absolute error MII
  "0",                                                        //Max. absolute error MIII
  "0",                                                        //Max. absolute error MIV
  "0",                                                        //Max. computation time MI
  "0",                                                        //Max. computation time MII
  "0",                                                        //Max. computation time MIII
  "0",                                                        //Max. computation time MIV
  "10\n100\n1000\n10000",                                     //Bounds
  "10\n20\n50\n100",                                          //Remainders
  "0.95\n0.90\n0.75\n0.50"                                    //Omega
};

const QString applicationName("ACETAF");
const QString version("Version 3.0.1");
const QString notAnalyticText("A gentle reminder:\n\nNot all functions are created analytic. Some are endowed by\ntheir creator with certain unacceptable flaws, among which\nare jumps, ambiguities and the pursuit of singularities.\n\nTo avoid a waste of valuable resources you are advised to\ncheck the analyticity of f. Better yet, let " + applicationName + " do the\njob for you (Hint: There's a button)!\n\n");

extern const QString descriptionText("The variable name in the function expression must be the literal 'z'.\n\nThe function expression may contain the arithmetic operators  +, -, *, /,  the imaginary unit 'i',  the functions\n\nsqr, sqrt, exp, ln, sin, cos, tan, cot, asin, acos, atan, acot, sinh, cosh, tanh, coth, asinh, acosh, atanh, acoth,   \n\nand the following functions with two arguments:  pow, power, root.");

const QString aboutText(applicationName + ", " + version + "\nusing CoStLy Version " + CoStLy::Version().c_str() + "\n\nCopyright (C) 2003-8\nMarkus Neher, markus.neher@math.uni-karlsruhe.de\nIngo Eble, ingoeble@web.de");

#ifdef FILIB_VERSION
const QString fileText("# *** " + applicationName + " " + version + " ***\n# *** CoStLy Version " + CoStLy::Version().c_str() + " ***\n# *** filib++ Version 2.0 as of 18-Jun-2004 ***");
#else //C-XSC Version
const QString fileText("# *** " + applicationName + " " + version + " ***\n# *** CoStLy Version " + CoStLy::Version().c_str() + " ***\n# *** C-XSC  Version 2.2.3 ***");
#endif

extern const QString fileIndicator(".dat");
extern const QString fileMask("*.dat");

MainWindow::MainWindow() : QMainWindow( 0, "acetaf" ), 
                                         calc(0), 
                                         acc(0), 
                                         bounds(0), 
                                         remainders(0), 
                                         omegare(0), 
                                         omegaiv(0), 
                                         garbage(0),
                                         filename("noName")
{
  printer  = new QPrinter();
  history  = new AHistory();

  this->setCaption( applicationName + ": " + filename );

  //Create Toolbar
  QPixmap 
    openIcon  ( fileopen     ),
    saveIcon  ( filesave     ),
    saveasIcon( filesaveas   ),
    printIcon ( fileprint    ),
    runIcon   ( startexec    ),
    stopIcon  ( stopexec     ),
    analyticityIcon  ( analyticity  ),
    taylorIcon  ( tayloronly  );
 
  tools = new QToolBar( this, "tools");
  fileOpen   = new QToolButton(openIcon,  "Open File",0,this,SLOT(loadFile()),tools,"open file");
  fileSave   = new QToolButton(saveIcon,  "Save File",0,this,SLOT(saveFile()),tools,"save file");
  fileSaveAs = new QToolButton(saveasIcon,"Save File As",0,this,SLOT(saveFileAs()),tools,"save file as");
  filePrint  = new QToolButton(printIcon,"Print File",0,this,SLOT(printFile()),tools,"print file");
  runC  = new QToolButton(runIcon,"Start Calculation",0,this,SLOT(run()),tools,"run Calculation");
  stopC = new QToolButton(stopIcon,"Stop Calculation",0,this,SLOT(stop()),tools,"stop Calculation");
  analyticityfunc = new QToolButton(analyticityIcon,"Test if expression is analytic",0,this,SLOT(acheck()),tools,"test function");
  tayloronlyfunc = new QToolButton(taylorIcon,"Compute Taylor coefficients only",0,this,SLOT(taylor()),tools,"Taylor coefficients");

  //Create Menubar
  file = new QPopupMenu();
  menuBar()->insertItem( "&File", file );

  file->insertItem( openIcon, "Open", this, SLOT(loadFile()), CTRL+Key_O, 1);
  file->insertItem( saveIcon, "Save", this, SLOT(saveFile()), CTRL+Key_S, 2);
  file->insertItem( saveasIcon, "Save As", this, SLOT(saveFileAs()), 0, 3);
  file->insertSeparator();
  file->insertItem( printIcon, "Print", this, SLOT(printFile()), CTRL+Key_P, 4);
  file->insertSeparator();
  file->insertItem( "Quit", qApp, SLOT(quit()), CTRL+Key_Q );

  calculation = new QPopupMenu();
  menuBar()->insertItem( "&Calculation", calculation );

  calculation->insertItem( runIcon, "Start", this, SLOT(run()), 0, 1);
  calculation->insertItem( stopIcon, "Stop", this, SLOT(stop()), 0);
  calculation->insertItem( analyticityIcon, "Analyticity check", this, SLOT(acheck()), 0, 2);
  calculation->insertItem( taylorIcon, "Taylor coefficients only", this, SLOT(taylor()), 0, 3);
  
  QPopupMenu *help = new QPopupMenu();
  menuBar()->insertItem( "&Help", help );

  help->insertItem( "f(z)", this, SLOT(description()), 0);
  help->insertSeparator();
  help->insertItem( "About", this, SLOT(about()), 0);

  //Create Mainwindow
  input = new InputWidget( this, "input");
  output = new OutputWindow( 0, filename );

  //Set Mainwindow
  this->setCentralWidget( input );
  statusBar()->message( "Ready", 2000 );
}

MainWindow::~MainWindow()
{
  delete printer;
  delete tools;
  delete history;
  if( calc    ) delete calc;
  if( bounds   ) delete bounds;
  if( remainders   ) delete remainders;
  if( omegare   ) delete omegare;
  if( omegaiv   ) delete omegaiv;
  if( acc     ) delete acc;
}

void MainWindow::loadFile()
{
  QFileDialog fileDialog( this );
  QString fn( fileDialog.getOpenFileName(0, fileMask, this) );
  if( !fn.isEmpty() )
    {
      QFile file( fn );
      if( !file.open( IO_ReadOnly ) )
      {
	statusBar()->message("Cannot open file", 2000);
	return;
      }
      
      QString regExpStr( "\\" + fileIndicator );
      fn.replace( QRegExp( regExpStr ), "" ); 
      filename = fn;
      
      std::vector<QString> data;
      QTextStream in( &file );
      while( !in.eof() )
	{
	  QString row = in.readLine();
	  if( row.isEmpty() || row.contains( '#', false ) ) continue;
	  if( row.contains( "Output:" , false ) ) break;
	  data.push_back( row );
	}
      file.close();
      
      if( data.size() >= 39) 
	{
	  input->setLineEditValue( ExprEdit  , data[ 0 ]);
	  input->setLineEditValue( CenterEdit, data[ 1 ]);
	  input->setLineEditValue( RadiusEdit, data[ 2 ]);
	  
	  int k = 8, l = 4, methods = data[3].toInt();
	  while( methods > 0 || l > 0 )
	    {
	      if( methods >= k )
		{
		  input->setCheckBoxValue( l, true);
		  methods -= k;
		}
	      else
		{
		  input->setCheckBoxValue( l, false);
		}		  
	      k >>= 1; l--;
	    }

	  input->setCheckBoxValue( 5, ( data[ 4 ] == "yes" ) );//Fixed subdivision yes/no
	  input->setCheckBoxValue( 6, ( data[ 5 ] == "yes" ) );//Optimization yes/no

	  //Data of Method I
	  input->setLineEditValue( MaxNoSeg1Edit, data[ 6 ]);
	  input->setLineEditValue( MinNoSeg1Edit, data[ 7 ]);
	  input->setLineEditValue( RelErr1Edit  , data[ 8 ]);
	  input->setLineEditValue( AbsErr1Edit  , data[ 9 ]);
	  input->setLineEditValue( MaxTime1Edit , data[ 10 ]);

	  //Data of Method II
	  input->setLineEditValue( OrderTaylMeth2Edit, data[ 11 ]);
	  input->setLineEditValue( PrecTayl2Edit     , data[ 12 ]);
	  input->setLineEditValue( Test2Edit         , data[ 13 ]);
	  input->setLineEditValue( MaxOrd2Edit       , data[ 14 ]);
	  input->setLineEditValue( MaxNoSeg2Edit     , data[ 15 ]);
	  input->setLineEditValue( MinNoSeg2Edit     , data[ 16 ]);
	  input->setLineEditValue( RelErr2Edit       , data[ 17 ]);
	  input->setLineEditValue( AbsErr2Edit       , data[ 18 ]);
	  input->setLineEditValue( MaxTime2Edit      , data[ 19 ]);

	  //Data of Method III
	  input->setLineEditValue( OrdDeriv3Edit, data[ 20 ]);
	  input->setLineEditValue( MaxNoSeg3Edit, data[ 21 ]);
	  input->setLineEditValue( MinNoSeg3Edit, data[ 22 ]);
	  input->setLineEditValue( RelErr3Edit  , data[ 23 ]);
	  input->setLineEditValue( AbsErr3Edit  , data[ 24 ]);
	  input->setLineEditValue( MaxTime3Edit , data[ 25 ]);
	  
	  //Data of Method IV
	  input->setLineEditValue( OrderTaylMeth4Edit, data[ 26 ]);
	  input->setLineEditValue( PrecTayl4Edit     , data[ 27 ]);
	  input->setLineEditValue( OrdDeriv4Edit     , data[ 28 ]);
	  input->setLineEditValue( Test4Edit         , data[ 29 ]);
	  input->setLineEditValue( MaxOrd4Edit       , data[ 30 ]);
	  input->setLineEditValue( MaxNoSeg4Edit     , data[ 31 ]);
	  input->setLineEditValue( MinNoSeg4Edit     , data[ 32 ]);
	  input->setLineEditValue( RelErr4Edit       , data[ 33 ]);
	  input->setLineEditValue( AbsErr4Edit       , data[ 34 ]);
	  input->setLineEditValue( MaxTime4Edit      , data[ 35 ]);

	  int ix = data.size() - 36;
	  QString str("");
	  for(int i=2; i<=ix; i++)
	    {
	      if( data[ 35+i ] == "R" )
		{
		  input->setBounds( str );
		  str = "";
		  continue;
		}
	      if( data[ 35+i ] == "O" ) 
		{
		  input->setRemainders( str );
		  str = "";
		  continue;
		}
	      str += data[ 35+i ] + "\n";
	    }
	  input->setOmega( str );
	}
      else
	{
	  statusBar()->message( "Not enough data in input file", 2000 );
	  return;
	}
      
      output->clear();
      output->setFilename( filename );
      setCaption( applicationName + ": " + filename + fileIndicator );
      statusBar()->message( "Loaded file " + filename + fileIndicator, 2000 );
    }
  else statusBar()->message( "Loading aborted", 2000 );
}

void MainWindow::saveFileAs()
{
  QFileDialog fileDialog( this );
  QString fn = fileDialog.getSaveFileName(0, fileMask, this);
  if ( !fn.isEmpty() ) 
    {
      QString regExpStr( "\\" + fileIndicator );
      fn.replace( QRegExp( regExpStr ), "" ); 
      filename = fn;
      this->saveFile();
    } 
  else statusBar()->message("Data not saved", 2000);
}

void MainWindow::saveFile()
{
  QFile file( filename + fileIndicator );
  if( !file.open( IO_WriteOnly ) )
    {
      statusBar()->message("Cannot write file", 2000);
      return;
    }
  
  QTextStream out( &file );
  out << fileText + "\n\n";
  out << "# Expression\n" + LineEditValue( ExprEdit ) + "\n\n";
  out << "# Center\n" + LineEditValue( CenterEdit ) + "\n\n";
  out << "# Radius\n" + LineEditValue( RadiusEdit ) + "\n\n";
  
  int methods = 0, k = 8, l = 4;
  while( l > 0 )
    {
      if( input->CheckBoxValue( l ) )
	{
	  methods += k;
	}
      k >>= 1; l--;
    }
  out << "# Methods\n"; 
  out << QString::number( methods );
  out << "\n\n";
  out << "# Method II and IV: Fixed subdivision\n";
  out << ((input->fixedsubdiv)?"yes\n\n":"no\n\n");  
  out << "# Method II and IV: Optimization\n";
  out << ((input->optimization)?"yes\n\n":"no\n\n");  
  //Save data of Method I
  out << "# Method I: Max. no. of segments \n" + LineEditValue( MaxNoSeg1Edit ) + "\n\n";
  out << "# Method I: Min. no. of segments \n" + LineEditValue( MinNoSeg1Edit ) + "\n\n";
  out << "# Method I: Max. relative error  \n" + LineEditValue( RelErr1Edit   ) + "\n\n";
  out << "# Method I: Max. absolute error  \n" + LineEditValue( AbsErr1Edit   ) + "\n\n";
  out << "# Method I: Max. computation time\n" + LineEditValue( MaxTime1Edit  ) + "\n\n";
  //Save data of Method II
  out << "# Method II: Order of Taylor polynomial\n"       + LineEditValue( OrderTaylMeth2Edit ) + "\n\n";
  out << "# Method II: Precision of Taylor coefficients\n" + LineEditValue( PrecTayl2Edit      ) + "\n\n";
  out << "# Method II: Test segments\n"                    + LineEditValue( Test2Edit          ) + "\n\n";
  out << "# Method II: Max. Taylor order\n"                + LineEditValue( MaxOrd2Edit        ) + "\n\n";
  out << "# Method II: Max. no. of segments \n"            + LineEditValue( MaxNoSeg2Edit      ) + "\n\n";
  out << "# Method II: Min. no. of segments \n"            + LineEditValue( MinNoSeg2Edit      ) + "\n\n";
  out << "# Method II: Max. relative error  \n"            + LineEditValue( RelErr2Edit        ) + "\n\n";
  out << "# Method II: Max. absolute error  \n"            + LineEditValue( AbsErr2Edit        ) + "\n\n";
  out << "# Method II: Max. computation time\n"            + LineEditValue( MaxTime2Edit       ) + "\n\n";
  //Save data of Method III
  out << "# Method III: Order of derivative\n"   + LineEditValue( OrdDeriv3Edit ) + "\n\n";
  out << "# Method III: Max. no. of segments \n" + LineEditValue( MaxNoSeg3Edit ) + "\n\n";
  out << "# Method III: Min. no. of segments \n" + LineEditValue( MinNoSeg3Edit ) + "\n\n";
  out << "# Method III: Max. relative error  \n" + LineEditValue( RelErr3Edit   ) + "\n\n";
  out << "# Method III: Max. absolute error  \n" + LineEditValue( AbsErr3Edit   ) + "\n\n";
  out << "# Method III: Max. computation time\n" + LineEditValue( MaxTime3Edit  ) + "\n\n";
  //Save data of Method IV
  out << "# Method IV: Order of Taylor polynomial\n"       + LineEditValue( OrderTaylMeth4Edit ) + "\n\n";
  out << "# Method IV: Precision of Taylor coefficients\n" + LineEditValue( PrecTayl4Edit      ) + "\n\n";
  out << "# Method IV: Order of derivative\n"              + LineEditValue( OrdDeriv4Edit      ) + "\n\n";
  out << "# Method IV: Test segments\n"                    + LineEditValue( Test4Edit          ) + "\n\n";
  out << "# Method IV: Max. Taylor order\n"                + LineEditValue( MaxOrd4Edit        ) + "\n\n";
  out << "# Method IV: Max. no. of segments \n"            + LineEditValue( MaxNoSeg4Edit      ) + "\n\n";
  out << "# Method IV: Min. no. of segments \n"            + LineEditValue( MinNoSeg4Edit      ) + "\n\n";
  out << "# Method IV: Max. relative error  \n"            + LineEditValue( RelErr4Edit        ) + "\n\n";
  out << "# Method IV: Max. absolute error  \n"            + LineEditValue( AbsErr4Edit        ) + "\n\n";
  out << "# Method IV: Max. computation time\n"            + LineEditValue( MaxTime4Edit       ) + "\n\n";

  out << "# Coefficients\nB\n" + (input->Bounds())     + "\n\n";
  out << "# Remainders\nR\n" + (input->Remainders()) + "\n\n";
  out << "# Omega\nO\n" + (input->Omega())      + "\n\n";

  out << QChar(12);
  out << "Output:\n\n" + fileText + "\n\n";

  for( int i = 0 ; i < output->numLines() ; i++ ) 
       {  
         out << output->textLine( i ) << endl;
       }
	  
  file.close();

  output->setFilename( filename );

  setCaption( applicationName + ": " + filename + fileIndicator );
  statusBar()->message( "Saved file " + filename + fileIndicator, 2000 );
}
 
void MainWindow::printFile()
{
  const int MARGIN = 10;
  int pageNo = 1;
  
  if( printer->setup(this) ) 
    {               
      statusBar()->message( "Printing..." );
      QPainter p;
      if( !p.begin( printer ) ) return; // paint on printer
      p.setFont( output->font() );
      int yPos = 0; // y position for each line
      QFontMetrics fm = p.fontMetrics();
      QPaintDeviceMetrics metrics( printer ); // need width/height
                                              // of printer surface
      for( int i = 0 ; i < output->numLines() ; i++ ) 
	{
	  if ( MARGIN + yPos > metrics.height() - MARGIN ) 
	    {
	      QString msg = "Printing (page " + QString::number( ++pageNo ) + ")...";
	      statusBar()->message( msg );
	      printer->newPage();             // no more room on this page
	      yPos = 0;                       // back to top of page
	    }
	  p.drawText( MARGIN, MARGIN + yPos,metrics.width(), fm.lineSpacing(),ExpandTabs | DontClip,output->textLine( i ) );
	  yPos = yPos + fm.lineSpacing();
	}
      p.end();                                // send job to printer
      
      statusBar()->message( "Printing completed", 2000 );
    }
}

void MainWindow::taylor()
{
  TaylorCoeffDialog *tcd = new TaylorCoeffDialog( this, 0, true );

  connect(tcd, SIGNAL(sendData           (const Complex&,int)), 
	       SLOT  (calcTaylorCoeffOnly(const Complex&,int)) );

  tcd->exec();
  delete tcd;
}

void MainWindow::calcTaylorCoeffOnly(const Complex& c,int n)
{
  if( input->LineEditValue( ExprEdit ) == "" )
    {
      QMessageBox::information(this, applicationName, wrongInputMessageText[ NFct ]);
      return;
    }

  std::string functionString( removeWhiteSpace( (input->LineEditValue( ExprEdit )).latin1() ));

  output->clear();
  output->show();

  //Check if the given function is analytic on the given domain
  if( !history->contain( APair( functionString, Circle( c, 0.0 ) ) ) )
    {
      output->setOutput( notAnalyticText );
    }

  try
    {
      MathExpr<CITaylor> citaylfunction( functionString, "z" );
      CITaylor taylorcoeff( citaylfunction( CITaylor( CInterval( c ), n ) ) );

      std::string temp;

      QString text("The first ");
      text += QString::number( n+1 );
      text += " taylor coefficients of\n\n";
      text += input->LineEditValue( ExprEdit );
      text += "\n\nat ";
      text += complex2QStr( c, "% .10lG" );
      text += ":\n\n";
      temp = "";
      temp << taylorcoeff;
      text += QString( temp.c_str() );

      output->setOutput( text );
    }
  catch(syntax_error serr) 
    {
      QString mess("Syntax error in\n'");
      mess += input->LineEditValue( ExprEdit );
      mess += "':\n";
      mess += serr.what();
      anErrorOccurred( mess );
    }
  catch(division_by_zero)
    {
      QString mess("Division by zero occured in the calculation.");
      anErrorOccurred( mess );
    }
  catch(function_not_defined)
    {
      QString mess("f is not analytic in the development point " + complex2QStr( c, "% .10lG" ) + ".");
      anErrorOccurred( mess );
    }
}

void MainWindow::calcFinished(const TaylorMemory<CInterval> *tm)
{
  output->setOutput( outputString( Tail, 0, tm ) );

  if( calc ) delete calc; 
  calc = 0;

  input->InputEnable( true );
  setToolButtonEnabled( true );
  setMenuItemEnabled( true );
  this->setCursor( arrowCursor );
  statusBar()->message( "Calculation terminated", 2000 );
}

void MainWindow::OutputActivated(const MethodBase* mb, const OptimResult* ores)
{
  switch( mb->identity() )
    {
    case Method1:
      {
	if( ores )
	  {
	    output->setOutput( outputString( Body_Method1, ores, 0 ) );
	  }
	else
	  {
	    output->setOutput( outputString( Head_Method1, 0, 0 ) );
	    if( garbage ) 
	      garbage->setGarbage( "----------------------------------------\nGarbage output of Method I:\n");
	  }
	break;
      }
    case Method2:
      {
	if( ores )
	  {
	    output->setOutput( outputString( Body_Method2, ores, 0 ) );
	  }
	else
	  {
	    output->setOutput( outputString( Head_Method2, 0, 0 ) );
	    if( garbage ) 
	      garbage->setGarbage( "----------------------------------------\nGarbage output of Method II:\n");
	  }
	break;
      }
    case Method3:
      {
	if( ores )
	  {
	    output->setOutput( outputString( Body_Method3, ores, 0 ) );
	  }
	else
	  {
	    output->setOutput( outputString( Head_Method3, 0, 0 ) );
	    if( garbage ) 
	      garbage->setGarbage( "----------------------------------------\nGarbage output of Method III:\n");
	  }
	break;
      }
    case Method4:
      {
	if( ores )
	  {
	    output->setOutput( outputString( Body_Method4, ores, 0 ) );
	  }
	else
	  {
	    output->setOutput( outputString( Head_Method4, 0, 0 ) );
	    if( garbage ) 
	      garbage->setGarbage( "----------------------------------------\nGarbage output of Method IV:\n");
	  }
	break;
      }
    }
}

void MainWindow::anErrorOccurred(const QString& text)
{
  if( calc ) delete calc;
  calc = 0;

  QMessageBox::information(this, applicationName, text);
  input->InputEnable( true );
  setToolButtonEnabled( true );
  setMenuItemEnabled( true );
  output->clear();
  this->setCursor( arrowCursor );
}

void MainWindow::getMessage(const QString& text)
{
  output->setOutput( text );
}

void MainWindow::run()
{
  input->InputEnable( false );
  setToolButtonEnabled( false );
  setMenuItemEnabled( false );
  this->setCursor( waitCursor );
  
  /*-------------------------------+
   | Read and check the input data |
   +-------------------------------*/
  //No function given
  if( input->LineEditValue( ExprEdit ) == "" )
    {
      QMessageBox::information(this, applicationName, wrongInputMessageText[ NFct ]);
      input->InputEnable( true );
      setToolButtonEnabled( true );
      setMenuItemEnabled( true );
      this->setCursor( arrowCursor );
      return;
    }

  //No center or radius given
  if( (input->LineEditValue( CenterEdit ) == "")||(input->LineEditValue( RadiusEdit ) == "") )
    {
      QMessageBox::information(this, applicationName, wrongInputMessageText[ INC ]);
      input->InputEnable( true );
      setToolButtonEnabled( true );
      setMenuItemEnabled( true );
      this->setCursor( arrowCursor );
      return;
    }

  //Check which method is chosen, read the given data and set initial values.
  bool method_chosen = false;
  for(int i=1; i<=4; i++)
    {
      bool chkV( input->CheckBoxValue(i) );
      method_chosen |= chkV;
      if( chkV ) 
	{
	  switch( (Method)i )
	    {
	    case Method1:
	      {
		if( (input->LineEditValue( MaxNoSeg1Edit ) == "")||
		    (input->LineEditValue( MinNoSeg1Edit ) == "")||
		    (input->LineEditValue( RelErr1Edit   ) == "")||
		    (input->LineEditValue( AbsErr1Edit   ) == "")||
		    (input->LineEditValue( MaxTime1Edit  ) == "")   )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ OP ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		break;
	      }
	    case Method2:
	      {
		if( (input->LineEditValue( OrderTaylMeth2Edit ) == "")||
		    (input->LineEditValue( PrecTayl2Edit      ) == "")   )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ MII ]);
		    input->InputEnable( true ); 
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->LineEditValue( OrderTaylMeth2Edit )).toInt() < 1 )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ TO ]);
		    input->InputEnable( true ); 
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->optimization)&&
		    ((input->LineEditValue( Test2Edit   ) == "")||
		     (input->LineEditValue( MaxOrd2Edit ) == "")   ))
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ MIIV ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->optimization)&&
		    ((input->LineEditValue( MaxOrd2Edit )).toInt() > 150 ) )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ MO ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->LineEditValue( MaxNoSeg2Edit ) == "")||
		    (input->LineEditValue( MinNoSeg2Edit ) == "")||
		    (input->LineEditValue( RelErr2Edit   ) == "")||
		    (input->LineEditValue( AbsErr2Edit   ) == "")||
		    (input->LineEditValue( MaxTime2Edit  ) == "")   )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ OP ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->optimization)&&
		    ((input->LineEditValue( Test2Edit )).toInt() > (input->LineEditValue( MaxNoSeg2Edit )).toInt()) )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ WS ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		break;
	      }
	    case Method3:
	      {
		if( (input->LineEditValue( OrdDeriv3Edit ) == "") )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ MIII ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->LineEditValue( MaxNoSeg3Edit ) == "")||
		    (input->LineEditValue( MinNoSeg3Edit ) == "")||
		    (input->LineEditValue( RelErr3Edit   ) == "")||
		    (input->LineEditValue( AbsErr3Edit   ) == "")||
		    (input->LineEditValue( MaxTime3Edit  ) == "")   )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ OP ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		break;
	      }
	    case Method4:
	      {
		if( (input->LineEditValue( OrderTaylMeth4Edit ) == "")||
		    (input->LineEditValue( PrecTayl4Edit      ) == "")||
		    (input->LineEditValue( OrdDeriv4Edit      ) == "")   )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ MIV ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->LineEditValue( OrderTaylMeth4Edit )).toInt() < 1 )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ TO ]);
		    input->InputEnable( true ); 
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->optimization)&&
		    ((input->LineEditValue( Test4Edit   ) == "")||
		     (input->LineEditValue( MaxOrd4Edit ) == "")  ))
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ MIIV ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->optimization)&&
		    ((input->LineEditValue( MaxOrd4Edit )).toInt() > 150 ) )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ MO ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->LineEditValue( MaxNoSeg4Edit ) == "")||
		    (input->LineEditValue( MinNoSeg4Edit ) == "")||
		    (input->LineEditValue( RelErr4Edit   ) == "")||
		    (input->LineEditValue( AbsErr4Edit   ) == "")||
		    (input->LineEditValue( MaxTime4Edit  ) == "")   )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ OP ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		if( (input->optimization)&&
		    ((input->LineEditValue( Test4Edit )).toInt() > (input->LineEditValue( MaxNoSeg4Edit )).toInt()) )
		  {
		    QMessageBox::information(this, applicationName, wrongInputMessageText[ WS ]);
		    input->InputEnable( true );
		    setToolButtonEnabled( true );
		    setMenuItemEnabled( true );
		    this->setCursor( arrowCursor );
		    return;
		  }
		break;
	      }
	    }
	}
    }
  //No method chosen
  if( !method_chosen )
    {
      QMessageBox::information(this, applicationName, wrongInputMessageText[ NMth ]);
      input->InputEnable( true );
      setToolButtonEnabled( true );
      setMenuItemEnabled( true );
      this->setCursor( arrowCursor );
      return;
    }
  
  //Read the given parameters and set initial values
  std::string functionString( removeWhiteSpace( (input->LineEditValue( ExprEdit )).latin1() ) );

  Complex center;
  double radius;

  (input->LineEditValue( CenterEdit )) >> center;
  (input->LineEditValue( RadiusEdit )) >> radius;

  Circle domain( center, radius );
  
  std::vector<OptimData*> odat;
  int method24( 0 );

  try
    {
      for(int i=1; i<=4; i++)
	{
	  if( input->CheckBoxValue(i) ) 
	    {
	      switch( (Method)i )
		{
		case Method1:
		  {
		    int maS( atoi( (input->LineEditValue( MaxNoSeg1Edit )).latin1() ) );
		    int miS( atoi( (input->LineEditValue( MinNoSeg1Edit )).latin1() ) );

		    double r,a,t;
#ifdef FILIB_VERSION
		    (input->LineEditValue( RelErr1Edit )) >> r; 
		    (input->LineEditValue( AbsErr1Edit )) >> a;
		    (input->LineEditValue( MaxTime1Edit )) >> t;
#else //C-XSC Version
		    cxsc::real rr,ra,rt;
		    std::string s13( (input->LineEditValue( RelErr1Edit )).latin1() );
		    s13 >> cxsc::SaveOpt >> cxsc::RndDown >> rr >> cxsc::RestoreOpt;
		    std::string s14( (input->LineEditValue( AbsErr1Edit )).latin1() ); 
		    s14 >> cxsc::SaveOpt >> cxsc::RndDown >> ra >> cxsc::RestoreOpt;
		    std::string s15( (input->LineEditValue( MaxTime1Edit )).latin1() ); 
		    s15 >> cxsc::SaveOpt >> cxsc::RndDown >> rt >> cxsc::RestoreOpt;
		    r = cxsc::_double(rr);
		    a = cxsc::_double(ra);
		    t = cxsc::_double(rt);
#endif
		    odat.push_back( new OptimData( new MethodOne( domain, functionString ), maS, miS, r, a, t ) );
		    break;
		  }
		case Method2:
		  {
		    int maS( atoi( (input->LineEditValue( MaxNoSeg2Edit )).latin1() ) );
		    int miS( atoi( (input->LineEditValue( MinNoSeg2Edit )).latin1() ) );
		    
		    double r,a,t;
#ifdef FILIB_VERSION
		    (input->LineEditValue( RelErr2Edit )) >> r;
		    (input->LineEditValue( AbsErr2Edit )) >> a;
		    (input->LineEditValue( MaxTime2Edit )) >> t;
#else //C-XSC Version
		    cxsc::real rr,ra,rt;
		    std::string s13( (input->LineEditValue( RelErr2Edit )).latin1() );
		    s13 >> cxsc::SaveOpt >> cxsc::RndDown >> rr >> cxsc::RestoreOpt;
		    std::string s14( (input->LineEditValue( AbsErr2Edit )).latin1() ); 
		    s14 >> cxsc::SaveOpt >> cxsc::RndDown >> ra >> cxsc::RestoreOpt;
		    std::string s15( (input->LineEditValue( MaxTime2Edit )).latin1() ); 
		    s15 >> cxsc::SaveOpt >> cxsc::RndDown >> rt >> cxsc::RestoreOpt;
		    r = cxsc::_double(rr);
		    a = cxsc::_double(ra);
		    t = cxsc::_double(rt);
#endif
		    int order    ( atoi( (input->LineEditValue( OrderTaylMeth2Edit )).latin1() ) );
		    int precision( atoi( (input->LineEditValue( PrecTayl2Edit      )).latin1() ) );

		    odat.push_back( new OptimData( new MethodTwo( domain, order, precision, functionString ), 
						       maS, 
						       miS, 
						       r, 
						       a, 
						       t    ) );
		    method24 += 1;
		    break;
		  }
		case Method3:
		  {
		    int maS( atoi( (input->LineEditValue( MaxNoSeg3Edit )).latin1() ) );
		    int miS( atoi( (input->LineEditValue( MinNoSeg3Edit )).latin1() ) );

		    double r,a,t;
#ifdef FILIB_VERSION
		    (input->LineEditValue( RelErr3Edit )) >> r;
		    (input->LineEditValue( AbsErr3Edit )) >> a;
		    (input->LineEditValue( MaxTime3Edit )) >> t;
#else //C-XSC Version
		    cxsc::real rr,ra,rt;
		    std::string s13( (input->LineEditValue( RelErr3Edit )).latin1() );
		    s13 >> cxsc::SaveOpt >> cxsc::RndDown >> rr >> cxsc::RestoreOpt;
		    std::string s14( (input->LineEditValue( AbsErr3Edit )).latin1() ); 
		    s14 >> cxsc::SaveOpt >> cxsc::RndDown >> ra >> cxsc::RestoreOpt;
		    std::string s15( (input->LineEditValue( MaxTime3Edit )).latin1() ); 
		    s15 >> cxsc::SaveOpt >> cxsc::RndDown >> rt >> cxsc::RestoreOpt;
		    r = cxsc::_double(rr);
		    a = cxsc::_double(ra);
		    t = cxsc::_double(rt);
#endif
		    int order( atoi( (input->LineEditValue( OrdDeriv3Edit )).latin1() ) );

		    odat.push_back( new OptimData( new MethodThree( domain, order, functionString ), 
						   maS, 
						   miS, 
						   r, 
						   a, 
						   t    ) );

		    break;
		  }
		case Method4:
		  {
		    int maS( atoi( (input->LineEditValue( MaxNoSeg4Edit )).latin1() ) );
		    int miS( atoi( (input->LineEditValue( MinNoSeg4Edit )).latin1() ) );

		    double r,a,t;
#ifdef FILIB_VERSION
		    (input->LineEditValue( RelErr4Edit )) >> r;
		    (input->LineEditValue( AbsErr4Edit )) >> a;
		    (input->LineEditValue( MaxTime4Edit )) >> t;
#else //C-XSC Version
		    cxsc::real rr,ra,rt;
		    std::string s13( (input->LineEditValue( RelErr4Edit )).latin1() );
		    s13 >> cxsc::SaveOpt >> cxsc::RndDown >> rr >> cxsc::RestoreOpt;
		    std::string s14( (input->LineEditValue( AbsErr4Edit )).latin1() ); 
		    s14 >> cxsc::SaveOpt >> cxsc::RndDown >> ra >> cxsc::RestoreOpt;
		    std::string s15( (input->LineEditValue( MaxTime4Edit )).latin1() ); 
		    s15 >> cxsc::SaveOpt >> cxsc::RndDown >> rt >> cxsc::RestoreOpt;
		    r = cxsc::_double(rr);
		    a = cxsc::_double(ra);
		    t = cxsc::_double(rt);
#endif
		    int deriveord( atoi( (input->LineEditValue( OrdDeriv4Edit      )).latin1() ) );
		    int order    ( atoi( (input->LineEditValue( OrderTaylMeth4Edit )).latin1() ) );
		    int precision( atoi( (input->LineEditValue( PrecTayl4Edit      )).latin1() ) );

		    odat.push_back( new OptimData( new MethodFour( domain, order, precision, functionString, deriveord ),
						       maS,
						       miS,
						       r, 
						       a,
						       t    ) );

		    method24 += 2;
		    break;
		  }
		}
	    }
	}
      
      //Read Lists
      if( bounds ) delete bounds;
      bounds = new std::vector<int>();
      QString listboxentry( input->Bounds() );
      QTextIStream *in = new QTextIStream( &listboxentry ); 
      while( !in->atEnd() )
	{
	  int i;
	  (*in) >> i;
	  bounds->push_back( i );
	}
      delete in;

      if( remainders ) delete remainders;
      remainders = new std::vector<int>();
      listboxentry = input->Remainders();
      in = new QTextIStream( &listboxentry ); 
      while( !in->atEnd() )
	{
	  int i;
	  (*in) >> i;
	  remainders->push_back( i );
	}
      delete in;

      if( omegare ) delete omegare;
      if( omegaiv ) delete omegaiv;

      omegare = new std::vector<double>();
      omegaiv = new std::vector<Interval>();
      listboxentry = input->Omega();
      in = new QTextIStream( &listboxentry ); 
      while( !in->atEnd() )
	{
	  double r;
	  Interval i;

	  QString temp( in->readLine() );

	  if( !temp.isEmpty() )
	    {
#ifdef FILIB_VERSION
	      temp >> r;
	      std::string s( temp.latin1() );
	      s >> i;
#else //C-XSC Version
	      temp >> r;
	      std::string s( (temp + "," + temp).latin1() );
	      s >> i;
#endif
	      omegare->push_back( r );
	      omegaiv->push_back( i );
	    }
	}
      delete in;

      /*---------------------+
       | Prepare calculation |
       +---------------------*/

      //Clear output window and display it
      output->clear();
      output->show();

      //Check if the given function is analytic on the given domain
      if( !history->contain( APair( functionString, domain ) ) )
	{
	  output->setOutput( notAnalyticText );
	}
      output->setOutput( outputString( Head, 0, 0 ) );
      
      //Create garbage-window if chosen
      if( input->verbose )
	{
	  garbage = new GarbageWindow(0, filename);
	  garbage->show();
	}
      
      //Prepare calculation
      if( method24 && (input->fixedsubdiv) )
	{
	  std::vector<OptTaylData*> otdat( 2 ); otdat[0] = 0; otdat[1] = 0;
	  if( method24 == 1 || method24 == 3 )
	    {
	      int s( atoi( (input->LineEditValue( Test2Edit       )).latin1() ) );
	      int maO( atoi( (input->LineEditValue( MaxOrd2Edit   )).latin1() ) );
	      int maS( atoi( (input->LineEditValue( MaxNoSeg2Edit )).latin1() ) );
	      otdat[ 0 ] = new OptTaylData( s, maS, maO, (input->optimization) );
	    }
	  if( method24 == 2 || method24 == 3 )
	    {
	      int s( atoi( (input->LineEditValue( Test4Edit       )).latin1() ) );
	      int maO( atoi( (input->LineEditValue( MaxOrd4Edit   )).latin1() ) );
	      int maS( atoi( (input->LineEditValue( MaxNoSeg4Edit )).latin1() ) );
	      otdat[ 1 ] = new OptTaylData( s, maS, maO, (input->optimization) );
	    }

	  calc = new Calculation( odat, otdat, input->verbose );      
	}
      else
	{
	  calc = new Calculation( odat, input->verbose );
	}
      connect(calc, SIGNAL(activateOutputfor(const MethodBase*,const OptimResult*)) , 
	            SLOT  (OutputActivated  (const MethodBase*,const OptimResult*))   );
      connect(calc, SIGNAL(calcFinished(const TaylorMemory<CInterval>*)   ), 
                    SLOT  (calcFinished(const TaylorMemory<CInterval>*) )  );
      connect(calc, SIGNAL(calcStopped()   ), 
	            SLOT  (calcStopped()   )  );
      connect(calc, SIGNAL(sendMessage(const QString&) ), 
	            SLOT  (getMessage (const QString&) )  );
      connect(calc, SIGNAL(sendError       (const QString&) ), 
	            SLOT  (anErrorOccurred (const QString&) )  );
      if( garbage )
	connect(calc,    SIGNAL(sendGarbage(const QString&) ), 
	        garbage, SLOT  (getGarbage (const QString&) )  );

      //Start calculation
      calc->start();
    }
  catch(syntax_error serr) 
    {
      QString mess("Syntax error in\n'");
      mess += input->LineEditValue( ExprEdit );
      mess += "':\n";
      mess += serr.what();
      anErrorOccurred( mess );
    }
  catch(division_by_zero)
    {
      QString mess("Division by zero occured in the calculation.");
      anErrorOccurred( mess );
    }
  catch(function_not_defined)
    {
      QString mess("f is not analytic on the complete domain given.");
      anErrorOccurred( mess );
    }
}

void MainWindow::stop()
{
  if( calc ) calc->stop();
  if( acc  )  acc->stop();
}

void MainWindow::acheck()
{
  input->InputEnable( false );
  setToolButtonEnabled( false );
  setMenuItemEnabled( false );
  this->setCursor( waitCursor );

  /*-------------------------------+
   | Read and check the input data |
   +-------------------------------*/
  //No function given
  if( input->LineEditValue( ExprEdit ) == "" )
    {
      QMessageBox::information(this, applicationName, wrongInputMessageText[ NFct ]);
      input->InputEnable( true );
      setToolButtonEnabled( true );
      setMenuItemEnabled( true );
      this->setCursor( arrowCursor );
      return;
    }

  //No center or radius given
  if( (input->LineEditValue( CenterEdit ) == "")||(input->LineEditValue( RadiusEdit ) == "") )
    {
      QMessageBox::information(this, applicationName, wrongInputMessageText[ INC ]);
      input->InputEnable( true );
      setToolButtonEnabled( true );
      setMenuItemEnabled( true );
      this->setCursor( arrowCursor );
      return;
    }

  //Read the parameters and prepare ACheckControl
  std::string functionString( removeWhiteSpace( (input->LineEditValue( ExprEdit )).latin1() ) );

  Complex center;
  double radius;

  (input->LineEditValue( CenterEdit )) >> center;
  (input->LineEditValue( RadiusEdit )) >> radius;

  Circle domain( center, radius );
  APair  ap( functionString, domain );
  
  try
    {
      if( history->contain( ap ) )
	{
	  acheckFinished( &ap );
	}
      else
	{
	  acc = new ACheckControl( history, domain, functionString );
	  connect(acc, SIGNAL(checkFinished (const APair*)   ), 
	               SLOT  (acheckFinished(const APair*) )    );
	  connect(acc, SIGNAL(checkStopped ()   ), 
		       SLOT  (acheckStopped()   )  );
	  
	  //Start analyticity check
	  acc->start();
	}
    }
  catch(syntax_error serr) 
    {
      QString mess("Syntax error in\n'");
      mess += input->LineEditValue( ExprEdit );
      mess += "':\n";
      mess += serr.what();
      anErrorOccurred( mess );
    }
}

void MainWindow::acheckFinished(const APair *ap)
{
  Complex center;
  double radius;

  (input->LineEditValue( CenterEdit )) >> center;
  (input->LineEditValue( RadiusEdit )) >> radius;

  Circle current_domain( center, radius ), domain( ap->getDomain() );

  if( domain.getRadius() == 0.0 )
    {
	QMessageBox::information(this,
				 applicationName,
				 "The given function is not analytic near the center of the actual domain.");
    }
  else
    if( current_domain <= domain )
      {
	QMessageBox::information(this, 
				 applicationName, 
				 "The given function has been found analytic on the disc under consideration.");
      }
    else
      {
	QString info("An evaluation error occured in the computation.\nThe given function may not be analytic on the disc under consideration.\nIt has been found analytic on the following disc:\nCenter = ");
	info += (input->LineEditValue( CenterEdit ));
	info += "\nRadius = ";

#ifdef FILIB_VERSION
	//Radius nach unten gerundet ausgeben
	QString pred_string = double2QStr( domain.getRadius(), 8, 8, RND_DOWN );
	info += pred_string;
#else //C-XSC Version
	std::string newRadius("");
	newRadius << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(domain.getRadius()) << cxsc::RestoreOpt;
	info += QString( newRadius.c_str() );
#endif

	info += "\n\nContinue with these values?";
	if( QMessageBox::information( this, applicationName, info, "&Yes", "&No") == 0 ) 
	  {
#ifdef FILIB_VERSION
	    input->setLineEditValue( 2, pred_string );
#else //C-XSC Version
	    input->setLineEditValue( 2, QString( newRadius.c_str() ) );
#endif
	  }
      }
  
  if( acc ) delete acc;
  acc = 0;
  
  input->InputEnable( true );
  setToolButtonEnabled( true );
  setMenuItemEnabled( true );
  this->setCursor( arrowCursor );
  statusBar()->message( "Analyticity check finished", 2000 );
}

void MainWindow::acheckStopped()
{
  if( acc ) delete acc;
  acc = 0;
  
  input->InputEnable( true );
  setToolButtonEnabled( true );
  setMenuItemEnabled( true );
  this->setCursor( arrowCursor );
  statusBar()->message( "Analyticity check aborted", 2000 );
}

void MainWindow::calcStopped()
{
  if( calc ) delete calc; 
  calc = 0;

  input->InputEnable( true );
  setToolButtonEnabled( true );
  setMenuItemEnabled( true );
  this->setCursor( arrowCursor );
  statusBar()->message( "Calculation aborted", 2000 );
}

void MainWindow::description()
{
  QMessageBox::information(this, applicationName, descriptionText);
}

void MainWindow::about()
{
  QMessageBox::about(this, applicationName, aboutText);
}

void MainWindow::setToolButtonEnabled(bool show)
{
  fileOpen->setEnabled( show );
  fileSave->setEnabled( show );
  fileSaveAs->setEnabled( show );
  filePrint->setEnabled( show );
  runC->setEnabled( show );
  analyticityfunc->setEnabled( show ); 
  tayloronlyfunc->setEnabled( show );
}

void MainWindow::setMenuItemEnabled(bool show)
{
  file->setItemEnabled( 1, show );
  file->setItemEnabled( 2, show );
  file->setItemEnabled( 3, show );
  file->setItemEnabled( 4, show );
  calculation->setItemEnabled( 1, show );
  calculation->setItemEnabled( 2, show );
  calculation->setItemEnabled( 3, show );
}

QString MainWindow::LineEditValue(int i)
{
  if( input->LineEditValue( i ) == "" ) return defaultValues[i];
  else return input->LineEditValue( i );
}

/*-------------------------------------------------------------------------+
 |  calculate P( a, b ) := ( a+1 )*( a+2 )*.....*( a+b ), P( a, 0 ) := 1   |
 +-------------------------------------------------------------------------*/
Interval P(int a,int b)
{
  Interval prod( 1.0 );
  Interval A( a );
  for(int k=1;k<=b;k++)
    {
      prod *= ( A + Interval( k ) );
    }
  return prod;
}

#ifndef FILIB_VERSION
#include "rmath.hpp" //For functions exp and ln.
#endif

/*-----------------------+
 | generate ouput string |
 +-----------------------*/
QString MainWindow::outputString(OutputFlag oF, 
				 const OptimResult* ores, 
				 const TaylorMemory<CInterval> *tm)
{
  QString out(""); 
#ifndef FILIB_VERSION
  std::string temp("");
#endif
  switch( oF )
    {
    case Head:
      {
	//Get Date
	time_t d = time(0);
	char* datum = ctime(&d);

	out  = QString( datum )             + "\nCauchy's estimate for \n\n";
	out += (input->LineEditValue( ExprEdit ))  + "\n\naccording to Method(s):  "; 
	if( input->M1 ) out += "I,";
	if( input->M2 ) out += "II,";
	if( input->M3 ) out += "III,";
	if( input->M4 ) out += "IV,";
	out += "\non circle with center ";
	out += (input->LineEditValue( CenterEdit ));
	out += " and radius ";
	out += (input->LineEditValue( RadiusEdit ));
	break;
      }
    case Head_Method1:
      {
	out += "-------------------------------------------------------------\n";
 	out += "Estimate from Method I:\n";
	out += "  --> Cauchy's estimate <-- ";
	break;
      }
    case Head_Method2:
      {
	out += "-------------------------------------------------------------\n";
	out += "Estimate from Method II:\n";
	out += "  --> Cauchy's estimate with Taylor polynomial <--\n";
	out += "Parameters:\n     Order of Taylor polynomial = ";
	out += (input->LineEditValue( OrderTaylMeth2Edit )) + "\n";
	out += "     Precision of Taylor coefficients = ";
	out += (input->LineEditValue( PrecTayl2Edit )) + "\n";
	if( input->fixedsubdiv && !(input->optimization) )
	  {
	    out += "\n     Fixed grid without order optimization of the Taylor polynomial.\n";
	  }
	break;
      }
    case Head_Method3:
      {
	out += "-------------------------------------------------------------\n";	
	out += "Estimate from Method III: \n";
	out += "  --> Cauchy's estimate for m-th derivative <--\n";
	out += "Parameters:\n     Order of derivative = ";
	out += (input->LineEditValue( OrdDeriv3Edit )) + "\n";
	break;
      }
    case Head_Method4:
      {
	out += "-------------------------------------------------------------\n";
	out += "Estimate from Method IV: \n";
	out += "  --> Cauchy's estimate for m-th derivative with Taylor polynomial <--\n";
	out += "Parameters:\n     Order of derivative = ";
	out += (input->LineEditValue( OrdDeriv4Edit )) + "\n";
	out += "     Order of Taylor polynomial = ";
	out += (input->LineEditValue( OrderTaylMeth4Edit )) + "\n";
	out += "     Precision of Taylor coefficients = ";
	out += (input->LineEditValue( PrecTayl4Edit )) + "\n";
	if( input->fixedsubdiv && !(input->optimization) )
	  {
	    out += "\n     Fixed grid without order optimization of the Taylor polynomial.\n";
	  }
	break;
      }
    case Body_Method1:
      {
	if( (ores->err) == PREC_ERROR )
	  {
	    out  = "The desired relative error of ";
	    out += (input->LineEditValue( RelErr1Edit ));
	    out += "\nfor the maximum of | f(z) |\n";
	    out += "on the circle has not been reached with the\n";
	    out += "maximal number of segments given.\n";
	    out += "    Relative error of the result:  ";
#ifdef FILIB_VERSION
	    out += double2QStr( (ores->relprec ), 10, 10, RND_UP );
#else //C-XSC Version
	    temp = "";
	    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->relprec ) << cxsc::RestoreOpt;
	    out += QString( temp.c_str() );
#endif
	    out += "\n";
	  }
	out += "\nThe circle was split into ";
	out += QString::number( ores->segments_sum );
	out += " segments.\n";
	out += "At most ";
	out += QString::number( ores->candidates );
	out += " segments may contain the maximum.\n";
	out += "Fineness of the subdivision = ";
#ifdef FILIB_VERSION
	out += double2QStr( (ores->subtlety ), 10, 10, RND_UP );
#else //C-XSC Version
	temp = "";
	temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->subtlety ) << cxsc::RestoreOpt;
	out += QString( temp.c_str() );
#endif
	out += "\n\n";

	//Bounds for the selected Taylor coefficients
	double radius;
	( input->LineEditValue( RadiusEdit ) ) >> radius;
	std::vector<int>::const_iterator first = bounds->begin(), last = bounds->end();
	while( first != last )
	  {
	    int ind( *first++ );
	    if( ind >= 0 )
	      {
		if( sup( ores->max ) > 0.0 )
		  {
#ifdef FILIB_VERSION
		    filib::Interval::precision(10);
#endif

		    Interval logrhs = (ln( Interval(sup( ores->max )) )-ind*ln( Interval(radius) ) ) / ln(Interval(10));

		    double zrhs( floor( sup(logrhs) ) );
		    double mrhs, mant;
#ifdef FILIB_VERSION
		    SUB_UP( mrhs, sup( logrhs ), zrhs );
		    mant = filib::q_exp(mrhs*filib::q_log(10));
#else //C-XSC Version
		    mrhs = cxsc::_double( cxsc::subup( logrhs.sup(), zrhs ) );
		    mant = cxsc::_double( cxsc::exp  (mrhs * cxsc::ln(cxsc::real(10)) ) );
#endif
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ");
#ifdef FILIB_VERSION
		    out += double2QStr( mant, 10, 10, RND_UP );
#else //C-XSC Version
		    temp = "";
		    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		    out += QString( temp.c_str() );
#endif
		    out += QString("E") 
		      + double2QStr( zrhs, "%+.0lf" ) 
		      + QString("\n");
		  }
		else
		  {
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ") 
		      + QString::number( 0 );
		  }
	      }
	  }
	//Bounds for all Taylor coefficients
	out += "\n     | a[ j ] | <= ";
#ifdef FILIB_VERSION
	out += double2QStr( sup( ores->max ), 10, 10, RND_UP );
#else //C-XSC Version
	temp = "";
	temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(sup( ores->max )) << cxsc::RestoreOpt;
	out += QString( temp.c_str() );
#endif
	out += " / ";
	out += (input->LineEditValue( RadiusEdit ));
	out += "^j , for all j \n";
	out += "\n     M( ";
	out += (input->LineEditValue( RadiusEdit ));
	out += " ) =  ";
	out += interval2QStr( (ores->max), 10 );	
	out += "\n\nSelected k-th order remainder series bounds at omega * radius:\n\n";
	out += "     k | omega          ";

	for(int i = 0; i < omegare->size(); i++)
	  {
	    out += double2QStr( (*omegare)[i], "%-12.12lg" ) + QString("  ");
	  }
	out += "\n\n";

	//Selected remainder bounds for radius * omega
	{ Interval logrhs;
          double zrhs, mrhs;

	  for(int k = 0; k < remainders->size(); k++) 
	    {
	      out += "     | R[ ";
	      out += int2QStr( (*remainders)[k] , "%4i" );
	      out += " ] |  <=  ";
	      for(int l = 0; l < omegaiv->size() ; l++) 
		{
		  logrhs = ( ln( Interval(sup( ores->max )) )+ ( (*remainders)[k] + 1 )*ln( (*omegaiv)[l] ) - ln( 1-(*omegaiv)[l] ) ) / ln(Interval(10));  

		  zrhs = floor( sup(logrhs) );
		  double mant;
#ifdef FILIB_VERSION
		  SUB_UP( mrhs, sup( logrhs ), zrhs );
		  mant = filib::q_exp(mrhs*filib::q_log(10));
		  out +=  double2QStr( mant, 6, 4, RND_UP );
#else //C-XSC Version
		  mrhs = cxsc::_double( cxsc::subup( sup( logrhs ), zrhs ) );
		  mant = cxsc::_double(cxsc::exp(mrhs*cxsc::ln(10)));
		  temp = "";
		  temp << cxsc::SaveOpt << cxsc::SetPrecision(6,4) << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		  out += QString( temp.c_str() );
#endif

		  out += QString("E") 
		    + double2QStr( zrhs, "%-+5.0lf" ) 
		    + QString("  ");
		}
	      out += "\n";
	      }
	}
	//Computation time
	out += "\n     Computation time: ";
	out += double2QStr( (ores->calctime), "% .10lf" );
	out += " sec.\n\n";
	out += "Parameters of the optimization algorithm:\n";
	out += "    Maximum number of segments = ";
	out += (input->LineEditValue( MaxNoSeg1Edit ));
	out += "\n    Minimum number of segments = ";
	out += (input->LineEditValue( MinNoSeg1Edit ));
	out += "\nTermination criteria:\n";
	out += "    Acceptable relative error = ";
	out += (input->LineEditValue( RelErr1Edit )); 
	out += "\n    Acceptable absolute error = ";
	out += (input->LineEditValue( AbsErr1Edit )); 
	out += "\n\n\n";
	break;
      }
    case Body_Method2:
      {	
	if( (ores->err) == PREC_ERROR )
	  {
	    out  = "The desired relative error of ";
	    out += (input->LineEditValue( RelErr2Edit ));
	    out += "\nfor the maximum of | f(z) - p(z) |\n";
	    out += "on the circle has not been reached with the\n";
	    out += "maximal number of segments given.\n";
	    out += "    Relative error of the result:  ";
#ifdef FILIB_VERSION
	    out += double2QStr( ores->relprec, 10, 10, RND_UP );
#else //C-XSC Version
	    temp = "";
	    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->relprec ) << cxsc::RestoreOpt;
	    out += QString( temp.c_str() );
#endif
	    out += "\n";
	  }
	out += "\nThe circle was split into ";
	out += QString::number( (ores->segments_sum) );
	out += " segments.\n";
	out += "At most ";
	out += QString::number( (ores->candidates) );
	out += " segments may contain the maximum.\n";
	out += "Fineness of the subdivision = ";
#ifdef FILIB_VERSION
	out += double2QStr( (ores->subtlety), 10, 10, RND_UP );
#else //C-XSC Version
	temp = "";
	temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->subtlety ) << cxsc::RestoreOpt;
	out += QString( temp.c_str() );
#endif
	out += "\n\n";

	//Bounds for the selected Taylor coefficients
	double radius;
	( input->LineEditValue( RadiusEdit ) ) >> radius;
	int order( ores->PolyordMth2 );
	std::vector<int>::const_iterator first = bounds->begin(), last = bounds->end();
	
	while( first != last )
	  {
	    int ind = (*first++);
	    if( (ind >= 0)&&(ind > order) )
	      {
		if( sup(ores->max) > 0.0 )
		  {
		    Interval logrhs = (ln( Interval(sup(ores->max)) )-ind*ln( Interval(radius) ) ) / ln(Interval(10));
		    double zrhs( floor( sup(logrhs) ) );
		    double mrhs, mant;
#ifdef FILIB_VERSION
		    SUB_UP( mrhs, sup(logrhs), zrhs );
		    mant = filib::q_exp(mrhs*filib::q_log(10));
#else //C-XSC Version
		    mrhs = cxsc::_double( cxsc::subup( sup(logrhs), zrhs ) );
		    mant = cxsc::_double( cxsc::exp(mrhs*cxsc::ln(10)) );
#endif
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ");
#ifdef FILIB_VERSION
		    out += double2QStr( mant, 10, 10, RND_UP );
#else //C-XSC Version
		    temp = "";
		    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		    out += QString( temp.c_str() );
#endif
		    out += QString("E")
		      + double2QStr( zrhs, "%+.0lf" )
		      + QString("\n");
		  }
		else
		  {
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ") 
		      + QString::number(0)
		      + QString("\n");
		  }
	      }
	  }
	//Bounds for all Taylor coefficients
	out += "\n     | a[ j ] | <= ";
#ifdef FILIB_VERSION
	out += double2QStr( sup( ores->max ), 10, 10, RND_UP );
#else //C-XSC Version
	temp = "";
	temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(sup( ores->max )) << cxsc::RestoreOpt;
	out += QString( temp.c_str() );
#endif
	out += " / ";
	out += (input->LineEditValue( RadiusEdit ));
	out += "^j , for all j > ";
	out += QString::number( order );
	out += "\n\n     N( ";
	out += (input->LineEditValue( RadiusEdit ));
	out += " ) = ";
	out += interval2QStr( (ores->max), 10 );
	out += "\n\nSelected k-th order remainder series bounds at omega * radius:\n\n";
	out += "     k | omega          ";

	for(int i = 0; i < omegare->size(); i++)
	  {
	    out += double2QStr( (*omegare)[i], "%-12.12lg" ) + QString("  ");
	  }
	out += "\n\n";

	//Selected remainder bounds for radius * omega
	{ Interval logrhs;
          double zrhs, mrhs;

	  for(int k = 0; k < remainders->size(); k++) 
	    {
	      if( (*remainders)[k] >= order )
		{
		  out += "     | R[ ";
		  out += int2QStr( (*remainders)[k], "%4i" );
		  out += " ] |  <=  ";
		  for(int l = 0; l < omegaiv->size() ; l++) 
		    {
		      logrhs = ( ln( Interval(sup( ores->max )) )+((*remainders)[k]+1)*ln( (*omegaiv)[l] ) - ln( 1-(*omegaiv)[l] ) ) / ln(Interval(10));  
		      zrhs = floor( sup(logrhs) );
		      double mant;
#ifdef FILIB_VERSION
		      SUB_UP( mrhs, sup( logrhs ), zrhs );
		      mant = filib::q_exp(mrhs*filib::q_log(10));
  		      out += double2QStr( mant, 6, 4, RND_UP );
#else //C-XSC Version
		      mrhs = cxsc::_double( cxsc::subup( sup( logrhs ), zrhs ) );
		      mant = cxsc::_double( cxsc::exp(mrhs*cxsc::ln(10)) );
		      temp = "";
		      temp << cxsc::SaveOpt << cxsc::SetPrecision(6,4) << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		      out += QString( temp.c_str() );
#endif
		      out += QString("E")
			+ double2QStr( zrhs, "%-+5.0lf" )
			+ QString("  ");
		    }
		  out += "\n";
		}
	    }
	}
	//Computation time
	out += "\n     Computation time: ";
	out += double2QStr( (ores->calctime), "% .8lf" );
	out += " sec.\n\n";
	out += "Parameters of the optimization algorithm:\n";
	out += "    Maximum number of segments = ";
	out += (input->LineEditValue( MaxNoSeg2Edit ));
	out += "\n    Minimum number of segments = ";
	out += (input->LineEditValue( MinNoSeg2Edit ));
	out += "\nTermination criteria:\n";
	out += "    Acceptable relative error = ";
	out += (input->LineEditValue( RelErr2Edit )); 
	out += "\n    Acceptable absolute error = ";
	out += (input->LineEditValue( AbsErr2Edit )); 
	out += "\n\n\n";
	break;
      }
    case Body_Method3:
      {
	if( (ores->err) == PREC_ERROR )
	  {
	    out  = "The desired relative error of ";
	    out += (input->LineEditValue( RelErr3Edit ));
	    out += "\nfor the maximum of | f^(";
	    out += (input->LineEditValue( OrdDeriv3Edit ));
	    out += ")(z) |\n";
	    out += "on the circle has not been reached with the\n";
	    out += "maximal number of segments given.\n";
	    out += "    Relative error of the result:  ";
#ifdef FILIB_VERSION
	    out += double2QStr( (ores->relprec), 10, 10, RND_UP );
#else //C-XSC Version
	    temp = "";
	    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->relprec ) << cxsc::RestoreOpt;
	    out += QString( temp.c_str() );
#endif
	    out += "\n";
	  }  
	out += "\nThe circle was split into ";
	out += QString::number( ores->segments_sum );
	out += " segments.\n";
	out += "At most ";
	out += QString::number( ores->candidates );
	out += " segments may contain the maximum.\n";
	out += "Fineness of the subdivision = ";
#ifdef FILIB_VERSION
	out += double2QStr( (ores->subtlety), 10, 10, RND_UP );
#else //C-XSC Version
	temp = "";
	temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->subtlety ) << cxsc::RestoreOpt;
	out += QString( temp.c_str() );
#endif
	out += "\n\n";

	//Bounds for the selected Taylor coefficients
	double radius;
	( input->LineEditValue( RadiusEdit ) ) >> radius;
	int order( atoi( (input->LineEditValue( OrdDeriv3Edit )).latin1() ) );
	Interval zaehler;
	if( sup( ores->max ) > 0.0 )
	  {
	    zaehler = ln( Interval(sup( ores->max )) ) + order * ln(Interval(radius));
	  }
	std::vector<int>::const_iterator first = bounds->begin(), last = bounds->end();

	while( first != last )
	  {
	    int ind = (*first++);
	    if( (ind >= 0)&&( ind >= order ) )
	      {
		if( sup( ores->max ) > 0.0 )
		  {
		    Interval nenner( ln( P( ind-order, order ) ) + ind * ln(Interval(radius)) );
		    Interval logrhs( (zaehler-nenner)/ ln(Interval(10)) );
		    double zrhs( floor( sup(logrhs)) );
		    double mrhs,mant;
#ifdef FILIB_VERSION
		    SUB_UP( mrhs, sup(logrhs), zrhs );
		    mant = filib::q_exp(mrhs*filib::q_log(10));
#else //C-XSC Version
		    mrhs = cxsc::_double( cxsc::subup( sup(logrhs), zrhs ) );
		    mant = cxsc::_double(cxsc::exp(mrhs*cxsc::ln(10)));
#endif
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ");
#ifdef FILIB_VERSION
		    out += double2QStr( mant, 10, 10, RND_UP );
#else //C-XSC Version
		    temp = "";
		    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		    out += QString( temp.c_str() );
#endif
		    out += QString("E")
		      + double2QStr( zrhs, "%+.0lf" )
		      + QString("\n");
		  }
		else
		  {
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ") 
		      + QString::number(0)
		      + QString("\n");
		  }
	      }
	  }
	//Bounds for all Taylor coefficients
	out += "\n";
	if( sup( ores->max ) > 0.0 )
	  {
	    Interval logzaehler( zaehler/ln(Interval(10)) );
	    double zrhs( floor( sup(logzaehler) ) );
	    double mrhs, mant;

	    out += QString("     | a[ j ] | <= ");

#ifdef FILIB_VERSION
	    SUB_UP( mrhs, sup(logzaehler), zrhs );
	    mant = filib::q_exp(mrhs*filib::q_log(10));
	    out += double2QStr( mant, 10, 10, RND_UP );
#else //C-XSC Version
	    mrhs = cxsc::_double( cxsc::subup( sup(logzaehler), zrhs ) );
	    mant = cxsc::_double( cxsc::exp(mrhs*cxsc::ln(10)) );
	    temp = "";
	    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
	    out += QString( temp.c_str() );
#endif
	    out += QString("E")
	      + double2QStr( zrhs, "%+.0lf" );
	  }
	else
	  {
	    out += "     | a[ j ] | <= ";
	    out += QString::number(0);
	  }
	out += " / [ P( j - ";
	out += (input->LineEditValue( OrdDeriv3Edit ));
	out += " , ";
	out += (input->LineEditValue( OrdDeriv3Edit ));
	out += " ) * ";
	out += (input->LineEditValue( RadiusEdit ));
	out += "^j ] , for all j >= ";
	out += (input->LineEditValue( OrdDeriv3Edit ));
  	out += "\n\n     U( ";
	out += (input->LineEditValue( RadiusEdit ));
	out += " ) = ";
	out += interval2QStr( (ores->max), 10 );
	out += "\n\nSelected k-th order remainder series bounds at omega * radius:\n\n";
	out += "     k | omega          ";

	for(int i = 0; i < omegare->size(); i++)
	  {
	    out += double2QStr( (*omegare)[i], "%-12.12lg" ) + QString("  ");
	  }
	out += "\n\n";

	//Selected remainder bounds for radius * omega
	{ 
	  Interval rhs, logrhs;
	  double zrhs, mrhs, mant;
	  int rsize( remainders->size() );

	  //Preperations for the evaluation of the remainder series
	  std::vector<Interval> z( (*remainders)[ rsize-1 ] + 2 );
	  z[0] = 0; z[1] = 1;
	  for(int k = 2; k <= (*remainders)[ rsize-1 ] + 1; k++)
	    z[k] = (z[k-1]+1/P(0,k))/k;
	  for(int k = 0; k < rsize; k++) 
	    {
	      if( (*remainders)[k] >= order-1 )
		{
		  out += "     | R[ ";
		  out += int2QStr( (*remainders)[k], "%4i" );
		  out += " ] |  <=  ";
		  //Loop for all values of omega
		  for(int l = 0; l < omegaiv->size() ; l++) 
		    {
		      rhs = exp( ( order - 1 ) * ln( 1 - (*omegaiv)[l] ) ) * ( z[order-1] - ln( 1 - (*omegaiv)[l] ) / P(0,order-1) ); 
		      if( (order & 1) == 0 ) //order is odd, so order-1 is even.
			rhs = -rhs;

		      //Horner's scheme 
		      Interval rhs2;
		      rhs2 = -(*omegaiv)[l]/(order-2)+z[2];
		      int auxsign( 1 );
		      for(int ll = 2; ll <= order-2; ll++)
			{
			  rhs2 = rhs2*(*omegaiv)[l]/(order-1-ll)-auxsign*z[ll+1];
			  auxsign *= -1;
			}

		      rhs -= rhs2;

		      if( (*remainders)[k] >= order )
			{
			  //Substract partial sum
			  Interval psum( 1 /  P( (*remainders)[k]-order, order ) );
			  for(int ll = (*remainders)[k]; ll >= order+1; ll--)
			    {
			      psum *= (*omegaiv)[l];
			      psum += 1 / P( ll-order, order );
			    }
			  psum *= exp( order * ln( (*omegaiv)[l] ) );
			  rhs -= psum;
			}

		      //Caution: rhs may be damaged by cancellation
		      //Therefore compute alternative upper bound
		      //We compute  order+1  leading summands of the 
		      //remainder series and use a geometric series bound 
		      //for the remainder of the remainder
		      //
		      //Sum  order+1 terms with Horner's scheme
		      Interval rhsupbd( ((*omegaiv)[l]*( (*remainders)[k]+1 ))/( (*remainders)[k]+order+1 ) + 1 );
		      for(int ll = order; ll >= 2; ll--)
			{ rhsupbd = rhsupbd*((*omegaiv)[l]*( (*remainders)[k]-order+ll ))/( (*remainders)[k]+ll ) + 1; }
		      rhsupbd *= exp( ( (*remainders)[k] + 1 ) * ln( (*omegaiv)[l] ) ) / P( (*remainders)[k] + 1 - order, order );
		      //Add geometric series bound
		      rhsupbd += exp( ( (*remainders)[k] + order + 2 ) * ln( (*omegaiv)[l] ) ) / ( P( (*remainders)[k]+2, order ) * ( 1 - (*omegaiv)[l] ) );
		      if( sup( rhs ) > sup( rhsupbd ) )
			rhs = rhsupbd;
		      //multiply rhs with U and r^order 
		      rhs*=exp( order*ln( Interval(radius) ) ) * sup( ores->max );
		      //Eliminate eventual numerical underflow
		      rhs = sup( rhs ) + 1.0e-307;
		      //Print rhs 
		      //temp = ""; 
		      //temp << rhs;
		      logrhs = ln( rhs ) / ln(Interval(10));  
		      zrhs = floor( sup(logrhs) );
#ifdef FILIB_VERSION
		      SUB_UP( mrhs, sup( logrhs ), zrhs );
		      mant = filib::q_exp(mrhs*filib::q_log(10));
      		      out += double2QStr( mant, 6, 4, RND_UP );
#else //C-XSC Version
		      mrhs = cxsc::_double(cxsc::subup( sup( logrhs ), zrhs ));
		      mant = cxsc::_double(cxsc::exp(mrhs*cxsc::ln(10)));
		      temp = "";
		      temp << cxsc::SaveOpt << cxsc::SetPrecision(6,4) << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		      out += QString( temp.c_str() );
#endif
		      out += QString("E")
			+ double2QStr( zrhs, "%-+5.0lf" )
			+ QString("  ");
		    }
		  out += "\n";
		}
	    }
	}
	//Computation time
	out += "\n     Computation time: ";
	out += double2QStr( (ores->calctime), "% .8lf" );
	out += " sec.\n\n";
	out += "Parameters of the optimization algorithm:\n";
	out += "    Maximum number of segments = ";
	out += (input->LineEditValue( MaxNoSeg3Edit ));
	out += "\n    Minimum number of segments = ";
	out += (input->LineEditValue( MinNoSeg3Edit ));
	out += "\nTermination criteria:\n";
	out += "    Acceptable relative error = ";
	out += (input->LineEditValue( RelErr3Edit )); 
	out += "\n    Acceptable absolute error = ";
	out += (input->LineEditValue( AbsErr3Edit )); 
	out += "\n\n\n";
	break;
      }
    case Body_Method4:
      {
	if( (ores->err) == PREC_ERROR )
	  {
	    out  = "The desired relative error of ";
	    out += (input->LineEditValue( RelErr4Edit ));
	    out += "\nfor the maximum of | f^(";
	    out += (input->LineEditValue( OrdDeriv4Edit ));
	    out += ")(z)- p(z) |\n";
	    out += "on the circle has not been reached with the\n";
	    out += "maximal number of segments given.\n";
	    out += "    Relative error of the result:  ";
#ifdef FILIB_VERSION
	    out += double2QStr( (ores->relprec), 10, 10, RND_UP );
#else //C-XSC Version
	    temp = "";
	    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->relprec ) << cxsc::RestoreOpt;
	    out += QString( temp.c_str() );
#endif
	    out += "\n";
	  }
	out += "\nThe circle was split into ";
	out += QString::number( ores->segments_sum );
	out += " segments.\n";
	out += "At most ";
	out += QString::number( ores->candidates );
	out += " segments may contain the maximum.\n";
	out += "Fineness of the subdivision = ";
#ifdef FILIB_VERSION
	out += double2QStr( (ores->subtlety), 10, 10, RND_UP );
#else //C-XSC Version
	temp = "";
	temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real( ores->subtlety ) << cxsc::RestoreOpt;
	out += QString( temp.c_str() );
#endif
	out += "\n\n";

	//Bounds for the selected Taylor coefficients
	double radius;
	( input->LineEditValue( RadiusEdit ) ) >> radius;
	int deriveord( atoi( (input->LineEditValue( OrdDeriv4Edit )).latin1() ) );
	int order    ( ores->PolyordMth4 );
	Interval zaehler;
	if( sup( ores->max ) > 0.0 )
	  {
	    zaehler = ln(Interval(sup( ores->max )))+deriveord*ln(Interval(radius));
	  }
	std::vector<int>::const_iterator first = bounds->begin(), last = bounds->end();

	while( first != last )
	  {
	    int ind = (*first++);
	    if( (ind >= 0)&&( ind > deriveord+order ) )
	      {
		if( sup( ores->max ) > 0.0 )
		  {
		    Interval nenner( ln( P( ind-deriveord, deriveord ) ) + ind * ln(Interval(radius)) );
		    Interval logrhs( (zaehler-nenner)/ ln(Interval(10)) );
		    double zrhs( floor( sup(logrhs) ) );
		    double mrhs, mant;
#ifdef FILIB_VERSION
		    SUB_UP( mrhs, sup(logrhs), zrhs );
		    mant = filib::q_exp(mrhs*filib::q_log(10));
#else //C-XSC Version
		    mrhs = cxsc::_double( cxsc::subup( sup(logrhs), zrhs ) );
		    mant = cxsc::_double( cxsc::exp(mrhs*cxsc::ln(10)) );
#endif
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ");
#ifdef FILIB_VERSION
		    out += double2QStr( mant, 10, 10, RND_UP );
#else //C-XSC Version
		    temp = "";
		    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		    out += QString( temp.c_str() );
#endif
		    out += QString("E")
		      + double2QStr( zrhs, "%+.0lf" )
		      + QString("\n");
		  }
		else
		  {
		    out += QString("     | a[ ") 
		      + int2QStr( ind , "%6i" )
		      + QString(" ] | <= ") 
		      + QString::number(0)
		      + QString("\n");
		  }
	      }
	  }
	//Bounds for all Taylor coefficients
	out += "\n";
	if( sup( ores->max ) > 0.0 )
	  {
	    Interval logzaehler( zaehler/ ln(Interval(10)) );
	    double zrhs( floor( sup(logzaehler) ) );
	    double mrhs, mant;

	    out += "     | a[ j ] | <= ";

#ifdef FILIB_VERSION
	    SUB_UP( mrhs, sup(logzaehler), zrhs );
	    mant = filib::q_exp(mrhs*filib::q_log(10));
	    out += double2QStr( mant, 6, 4, RND_UP );
#else //C-XSC Version
	    mrhs = cxsc::_double( cxsc::subup( sup(logzaehler), zrhs ) );
	    mant = cxsc::_double( cxsc::exp(mrhs*cxsc::ln(10)) );
	    temp = "";
	    temp << cxsc::SaveOpt << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
	    out += QString( temp.c_str() );
#endif
	    out += QString("E")
	      + double2QStr( zrhs, "%+.0lf" );
	  }
	else
	  {
	    out += "     | a[ j ] | <= ";
	    out += QString::number( 0 );
	  }
	out += " / [ P( j - ";
	out += (input->LineEditValue( OrdDeriv4Edit ));
	out += " , ";
	out += (input->LineEditValue( OrdDeriv4Edit ));
	out += " ) * ";
	out += (input->LineEditValue( RadiusEdit ));
	out += "^j ] , for all j > ";
	out += QString::number( deriveord+order );
	out += "\n\n     V( ";
	out += (input->LineEditValue( RadiusEdit ));
	out += " ) = ";
	out += interval2QStr( (ores->max), 10 );
	out += "\n\nSelected k-th order remainder series bounds at omega * radius:\n\n";
	out += "     k | omega          ";

	for(int i = 0; i < omegare->size(); i++)
	  {
	    out += double2QStr( (*omegare)[i], "%-12.12lg" ) + QString("  ");
	  }
	out += "\n\n";

	//Selected remainder bounds for radius * omega
	{ 
	  Interval rhs, logrhs;
	  double zrhs, mrhs;
	  int rsize( remainders->size() );

	  //Preperations for the evaluation of the remainder series
	  std::vector<Interval> z( (*remainders)[ rsize-1 ] + 2 );
	  z[0] = 0; z[1] = 1;
	  for(int k = 2; k <= (*remainders)[ rsize-1 ] + 1; k++)
	    z[k] = (z[k-1]+1/P(0,k))/k;
	  for(int k = 0; k < rsize; k++) 
	    {
	      if( (*remainders)[k] >= deriveord+order )
		{
		  out += "     | R[ ";
		  out += int2QStr( (*remainders)[k], "%4i" );
		  out += " ] |  <=  ";
		  //Loop for all values of omega
		  for(int l = 0; l < omegaiv->size() ; l++) 
		    {
		      //Here, we always have cancellation
		      //Therefore we compute  deriveord+1  
		      //leading summands of the remainder series 
		      //and use a geometric series bound 
		      //for the remainder of the remainder
		      //
		      //Sum  deriveord+1 terms with Horner's scheme
		      Interval rhsupbd( ((*omegaiv)[l]*( (*remainders)[k]+1 ))/( (*remainders)[k]+deriveord+1 ) + 1 );
		      for(int ll = deriveord; ll >= 2; ll--)
			{ rhsupbd = rhsupbd*((*omegaiv)[l]*( (*remainders)[k]-deriveord+ll ))/( (*remainders)[k]+ll ) + 1; }
		      rhsupbd *= exp( ( (*remainders)[k] + 1 ) * ln( (*omegaiv)[l] ) ) / P( (*remainders)[k] + 1 - deriveord, deriveord );
		      //Add geometric series bound
		      rhsupbd += exp( ( (*remainders)[k] + deriveord + 2 ) * ln( (*omegaiv)[l] ) ) / ( P( (*remainders)[k]+2, deriveord ) * ( 1 - (*omegaiv)[l] ) );
		      rhs = rhsupbd;
		      //multiply rhs with V * r^deriveord 
		      rhs*=exp( deriveord*ln( Interval(radius) ) ) * sup( ores->max );
		      //Eliminate eventual numerical underflow
		      rhs = sup( rhs ) + 1.0e-307;
		      //Print rhs 
		      //temp = ""; 
		      //temp << rhs;
		      logrhs = ln( rhs ) / ln(Interval(10));  
		      zrhs = floor( sup(logrhs) );
		      double mant;
#ifdef FILIB_VERSION
		      SUB_UP( mrhs, sup( logrhs ), zrhs );
		      mant = filib::q_exp(mrhs*filib::q_log(10));
      		      out += double2QStr( mant, 6, 4, RND_UP );
#else //C-XSC Version
		      mrhs = cxsc::_double( cxsc::subup( sup( logrhs ), zrhs ) );
		      mant = cxsc::_double( cxsc::exp(mrhs*cxsc::ln(10)) );
		      temp = "";
		      temp << cxsc::SaveOpt << cxsc::SetPrecision(6,4) << cxsc::RndUp << cxsc::real(mant) << cxsc::RestoreOpt;
		      out += QString( temp.c_str() );
#endif
		      out += QString("E")
			+ double2QStr( zrhs, "%-+5.0lf" )
			+ QString("  ");
		    }
		  out += "\n";
		}
	    }
	}

	//Computation time
	out += "\n     Computation time: ";
	out += double2QStr( (ores->calctime), "% .8lf" );
	out += " sec.\n\n";
	out += "Parameters of the optimization algorithm:\n";
	out += "    Maximum number of segments = ";
	out += (input->LineEditValue( MaxNoSeg4Edit ));
	out += "\n    Minimum number of segments = ";
	out += (input->LineEditValue( MinNoSeg4Edit ));
	out += "\nTermination criteria:\n";
	out += "    Acceptable relative error = ";
	out += (input->LineEditValue( RelErr4Edit )); 
	out += "\n    Acceptable absolute error = ";
	out += (input->LineEditValue( AbsErr4Edit )); 
	out += "\n\n\n";
	break;
      }
    case Tail:
      {
	if( tm )
	  {
	    out = "-------------------------------------------------------------\n";
	    out += "Computed Taylor coefficients:\n\n";

	    std::string tmp = "";
	    tmp << (tm->getTaylor());

	    out += QString( tmp.c_str() );
	    out += "\n";
	  }
	break;
      }
    }
  return out;
}	

#include "gui_mainwindow.moc"

/*

  End of File: gui_mainwindow.cpp

*/
