/*

 File: gui_outputwindow.cpp, 2003/06/17

 ACETAF, Version 3.0.1

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "gui_outputwindow.h"

#include <qmultilinedit.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qapplication.h>
#include <qregexp.h> 
#include <qstatusbar.h>

extern const QString fileIndicator;
extern const QString fileMask;

OutputWindow::OutputWindow(QWidget *parent, const char* name) : QMainWindow(parent, name),
							        filename   ( name       )
{
  QString caption( "Output of file " + filename );

  this->setCaption( caption + fileIndicator );

  //Create Menubar
  window = new QPopupMenu();
  menuBar()->insertItem( "&Window", window );

  window->insertItem( "Close", this, SLOT(closeFile()));

  //Create Mainwindow
  edit = new QMultiLineEdit( this, "output");

  edit->setMinimumSize(300, 400);
  edit->setReadOnly( true );
  QFont font( "Courier", 14 );
  font.setFixedPitch( true );
  edit->setFont( font );

  //Set Mainwindow
  this->setCentralWidget( edit );
}

OutputWindow::~OutputWindow()
{
}

void OutputWindow::setFilename(const QString& fn)
{
  QString caption( "Output of file " + fn );
  this->setCaption( caption + fileIndicator );
}

void OutputWindow::clear()
{
  edit->clear();
}

QString OutputWindow::text()
{
  return edit->text();
}

QString OutputWindow::textLine(int i)
{
  return edit->textLine( i );
}

int OutputWindow::numLines()
{
  return edit->numLines();
}

void OutputWindow::closeFile()
{
  this->close();
}

void OutputWindow::setOutput(const QString& text)
{
  edit->setAutoUpdate( false );
  edit->append( text );
  edit->setAutoUpdate( true );
  edit->setCursorPosition ( edit->numLines(), 1);
  edit->repaint();
}

#include "gui_outputwindow.moc"

/*

  End of File: gui_outputwindow.cpp

*/

	  
	    



