/*

 File: stringutil.cpp, 2004/09/30

 ACETAF, Version 3.0.1

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "stringutil.h"
#include "error.h"
#include "adaptintval.h"
#include "cinterval.h"

#include <cstdio>
#include <sstream>

void operator >>(const QString& s,int&    i) { i = s.toInt();         } 
void operator >>(const QString& s,double& d) { d = s.toDouble();      } 
void operator >>(const QString& s,Complex& c)
{
  double r=0, i=0;
  sscanf( s.latin1(), "%lG%*[^0-9+-]%lG", &r, &i );
  c = Complex( r, i );
}
void operator >>(const QString& s,Interval& i)
{
  std::istringstream in( std::string( s.latin1() ) );
  in >> i;
}
void operator >>(const QString& s,CInterval& c)
{
  std::istringstream in( std::string( s.latin1() ) );
  in >> c;
}
QString complex2QStr(const Complex& c,const std::string& format)
{
  return ( QString("(") + double2QStr( c.real(), format ) + QString(",") + double2QStr( c.imag(), format ) + QString(")") );
}
QString interval2QStr(const Interval& iv, unsigned int prec)
{
  std::ostringstream out;
#ifdef FILIB_VERSION
  int oprec = filib::Interval::precision( prec );
  out << iv;
  filib::Interval::precision( oprec );
#else
  out << cxsc::SaveOpt << cxsc::SetPrecision( prec, prec ) << iv << cxsc::RestoreOpt;
#endif

  return QString( out.str().c_str() );
}
QString double2QStr(const double& d,int wdth, int prec,int rnd)
{
  std::ostringstream os;
  os.setf( std::ios::fixed );
  os.width(wdth);
  os.precision(prec);

#ifdef FILIB_VERSION
  double a,b,c;
  a = filib::q_lg10( std::abs(d) );
  if( a < 0.0 ) a -= 1;
  b = int(a);
  c = filib::q_ex10( b - prec );
#endif	

  switch( rnd )
    {
    case RND_DOWN:
      {
#ifdef FILIB_VERSION
	os << (d - c);
#endif	
	break;
      }
    case RND_NEAR:
      {
#ifdef FILIB_VERSION
	os << d;
#endif	
	break;
      }
    case RND_UP:
      {
#ifdef FILIB_VERSION
	os << (d + c);
#endif	
	break;
      }
    }

  return QString( (os.str()).c_str() );
}
QString double2QStr(const double& d,const std::string& format)
{
  static char tmp[256];
  sprintf( tmp, format.c_str(), d ); 
  return QString( tmp );
}
QString int2QStr(int i,const std::string& format)
{
  static char tmp[256];
  sprintf( tmp, format.c_str(), i ); 
  return QString( tmp );
}
std::string getline(std::istream& is)
{
  std::string line("");
  char ch;
  
  while( is.get(ch) && isspace(ch) ) {} //Skip leading blanks
  is.putback(ch);
  while( is.get(ch) && (ch !='#') && (ch != '\n') ) line+=ch;
  if( ch == '#' ) while( is.get(ch) && (ch != '\n') ); //Read until end of line
  
  return line;
}
std::string removeWhiteSpace(const std::string& s)
{
  std::string t("");
  std::string::const_iterator first = s.begin(),
                              last  = s.end();

  char ch;
  while( first != last )
    {
      ch = (*first++);         //The following ASCII characters will be removed:
      if( !( ch == char( 9) || //  TAB,
	     ch == char(10) || //  LF,
	     ch == char(11) || //  VT,
	     ch == char(12) || //  FF,
	     ch == char(13) || //  CR, and the Space character.
	     ch == char(32)    ) )  t += ch;
    }
  return t;
}

/*

  End of File: stringutil.cpp

*/
